searchData={"items":[{"type":"module","title":"LiveVue","doc":"LiveVue provides seamless integration between Phoenix LiveView and Vue.js components.","ref":"LiveVue.html"},{"type":"module","title":"Installation and Configuration - LiveVue","doc":"See README.md for installation instructions and usage.","ref":"LiveVue.html#module-installation-and-configuration"},{"type":"module","title":"Component Options - LiveVue","doc":"When using the `vue/1` component or `~V` sigil, the following options are supported:","ref":"LiveVue.html#module-component-options"},{"type":"module","title":"Required Attributes - LiveVue","doc":"* `v-component` (string) - Name of the Vue component (e.g., \"YourComponent\", \"directory/Example\")\n\n> #### Tip {: .tip}\n>\n> Value of `v-component` will be directly passed to `resolve` function of the `createLiveVue` instance.\n> It should return Vue component or a promise that resolves to a Vue component.\n> In a standard setup, you can find it in `assets/vue/index.js`.","ref":"LiveVue.html#module-required-attributes"},{"type":"module","title":"Optional Attributes - LiveVue","doc":"* `id` (string) - Explicit ID of the wrapper component. If not provided, a random one will be\n    generated. Useful to keep ID consistent in development (e.g., \"vue-1\")\n  * `class` (string) - CSS class(es) to apply to the Vue component wrapper\n    (e.g., \"my-class\" or \"my-class another-class\")\n  * `v-ssr` (boolean) - Whether to render the component on the server. Defaults to the value set\n    in config (default: true)\n  * `v-socket` (LiveView.Socket) - LiveView socket, should be provided when rendering inside LiveView","ref":"LiveVue.html#module-optional-attributes"},{"type":"module","title":"Event Handlers - LiveVue","doc":"* `v-on:*` - Vue event handlers can be attached using the `v-on:` prefix\n    (e.g., `v-on:click`, `v-on:input`)","ref":"LiveVue.html#module-event-handlers"},{"type":"module","title":"Props and Slots - LiveVue","doc":"* All other attributes are passed as props to the Vue component\n  * Slots can be passed as regular Phoenix slots","ref":"LiveVue.html#module-props-and-slots"},{"type":"macro","title":"LiveVue.sigil_VUE/2","doc":"Inlines a Vue single-file component inside a LiveView. This is the new recommended way over the `~V` sigil.","ref":"LiveVue.html#sigil_VUE/2"},{"type":"function","title":"LiveVue.vue/1","doc":"Renders a Vue component within Phoenix LiveView.","ref":"LiveVue.html#vue/1"},{"type":"function","title":"Examples - LiveVue.vue/1","doc":"<.vue\n      v-component=\"MyComponent\"\n      message=\"Hello\"\n      v-on:click=\"handleClick\"\n      class=\"my-component\"\n    />\n\n    <.vue\n      v-component=\"nested/Component\"\n      v-ssr={false}\n      items={@items}\n    >\n      <:default>Default slot content</:default>\n      <:named>Named slot content</:named>\n    </.vue>","ref":"LiveVue.html#vue/1-examples"},{"type":"module","title":"LiveVue.Components","doc":"Macros to improve the developer experience of crossing the Liveview/Vue boundary.","ref":"LiveVue.Components.html"},{"type":"macro","title":"LiveVue.Components.__using__/1","doc":"Generates functions local to your current module that can be used to render Vue components.\nTODO: This could perhaps be optimized to only read the files once per compilation.","ref":"LiveVue.Components.html#__using__/1"},{"type":"macro","title":"Examples - LiveVue.Components.__using__/1","doc":"```elixir\nuse LiveVue.Components, vue_root: [\"./assets/vue\", \"./lib/my_app_web\"]\n```","ref":"LiveVue.Components.html#__using__/1-examples"},{"type":"protocol","title":"LiveVue.Encoder","doc":"Protocol for encoding values to JSON for LiveVue.\n\nThis protocol is used to safely transform structs into plain maps before\ncalculating JSON patches. It ensures that struct fields are explicitly\nexposed and prevents accidental exposure of sensitive data.\n\nIt's very similar to Jason.Encoder, but it's converting structs to maps instead of strings.","ref":"LiveVue.Encoder.html"},{"type":"protocol","title":"Deriving - LiveVue.Encoder","doc":"The protocol allows leveraging Elixir's `@derive` feature to simplify protocol\nimplementation in trivial cases. Accepted options are:\n\n* `:only` - encodes only values of specified keys.\n* `:except` - encodes all struct fields except specified keys.\n\nBy default all keys except the `:__struct__` key are encoded.","ref":"LiveVue.Encoder.html#module-deriving"},{"type":"protocol","title":"Example - LiveVue.Encoder","doc":"Let's assume a presence of the following struct:\n\n    defmodule User do\n      defstruct [:name, :email, :password]\n    end\n\nIf we were to call `@derive LiveVue.Encoder` just before `defstruct`, an\nimplementation would be generated that encodes all fields except `:__struct__`:\n\n    defmodule User do\n      @derive LiveVue.Encoder\n      defstruct [:name, :email, :password]\n    end\n\nIf we called `@derive {LiveVue.Encoder, only: [:name, :email]}`, only the\nspecified fields would be encoded:\n\n    defmodule User do\n      @derive {LiveVue.Encoder, only: [:name, :email]}\n      defstruct [:name, :email, :password]\n    end\n\nIf we called `@derive {LiveVue.Encoder, except: [:password]}`, all fields\nexcept the specified ones would be encoded:\n\n    defmodule User do\n      @derive {LiveVue.Encoder, except: [:password]}\n      defstruct [:name, :email, :password]\n    end","ref":"LiveVue.Encoder.html#module-example"},{"type":"protocol","title":"Deriving outside of the module - LiveVue.Encoder","doc":"If you don't own the struct you want to encode, you may use Protocol.derive/3 placed outside of any module:\n\nProtocol.derive(LiveVue.Encoder, User, only: [...])","ref":"LiveVue.Encoder.html#module-deriving-outside-of-the-module"},{"type":"protocol","title":"Custom implementations - LiveVue.Encoder","doc":"You may define your own implementation for the struct:\n\ndefimpl LiveVue.Encoder, for: User do\n  def encode(struct, opts) do\n    struct\n    |> Map.take([:first, :second])\n    |> LiveVue.Encoder.encode(opts)\n  end\nend","ref":"LiveVue.Encoder.html#module-custom-implementations"},{"type":"function","title":"LiveVue.Encoder.encode/2","doc":"Encodes a value to one of the primitive types.","ref":"LiveVue.Encoder.html#encode/2"},{"type":"type","title":"LiveVue.Encoder.opts/0","doc":"","ref":"LiveVue.Encoder.html#t:opts/0"},{"type":"type","title":"LiveVue.Encoder.t/0","doc":"","ref":"LiveVue.Encoder.html#t:t/0"},{"type":"module","title":"LiveVue.Reload","doc":"Utilities for easier integration with Vite in development","ref":"LiveVue.Reload.html"},{"type":"function","title":"LiveVue.Reload.vite_assets/1","doc":"Renders the vite assets in development, and in production falls back to normal compiled assets","ref":"LiveVue.Reload.html#vite_assets/1"},{"type":"function","title":"Attributes - LiveVue.Reload.vite_assets/1","doc":"* `assets` (`:list`) (required)","ref":"LiveVue.Reload.html#vite_assets/1-attributes"},{"type":"function","title":"Slots - LiveVue.Reload.vite_assets/1","doc":"* `inner_block` (required) - what should be rendered when Vite path is not defined.","ref":"LiveVue.Reload.html#vite_assets/1-slots"},{"type":"behaviour","title":"LiveVue.SSR","doc":"A behaviour for rendering Vue components server-side.\n\nTo define a custom renderer, change the application config in `config.exs`:\n\n    config :live_vue, ssr_module: MyCustomSSRModule\n\nExposes a telemetry span for each render under key `[:live_vue, :ssr]`","ref":"LiveVue.SSR.html"},{"type":"type","title":"LiveVue.SSR.component_name/0","doc":"","ref":"LiveVue.SSR.html#t:component_name/0"},{"type":"type","title":"LiveVue.SSR.props/0","doc":"","ref":"LiveVue.SSR.html#t:props/0"},{"type":"function","title":"LiveVue.SSR.render/3","doc":"","ref":"LiveVue.SSR.html#render/3"},{"type":"callback","title":"LiveVue.SSR.render/3","doc":"","ref":"LiveVue.SSR.html#c:render/3"},{"type":"type","title":"LiveVue.SSR.render_response/0","doc":"A render response which should have shape\n\n%{\n  html: string,\n  preloadLinks: string\n}","ref":"LiveVue.SSR.html#t:render_response/0"},{"type":"type","title":"LiveVue.SSR.slots/0","doc":"","ref":"LiveVue.SSR.html#t:slots/0"},{"type":"module","title":"LiveVue.SSR.NodeJS","doc":"Implements SSR by using `NodeJS` package.\n\nUnder the hood, it invokes \"render\" function exposed by `server.js` file.\nYou can see how `server.js` is created by looking at `assets.deploy` command\nand `package.json` build-server script.","ref":"LiveVue.SSR.NodeJS.html"},{"type":"function","title":"LiveVue.SSR.NodeJS.render/3","doc":"Callback implementation for  LiveVue.SSR.render/3 .","ref":"LiveVue.SSR.NodeJS.html#render/3"},{"type":"function","title":"LiveVue.SSR.NodeJS.server_path/0","doc":"","ref":"LiveVue.SSR.NodeJS.html#server_path/0"},{"type":"module","title":"LiveVue.SSR.ViteJS","doc":"Implements SSR by making a POST request to `http://{:vite_host}/ssr_render`.\n\n`ssr_render` is implemented as a Vite plugin. You have to add it to the `vite.config.js` plugins section.\n\n```javascript\nimport liveVuePlugin from \"live_vue/vitePlugin\"\n\n{\n  publicDir: \"static\",\n  plugins: [vue(), liveVuePlugin()],\n  // ...\n}\n```","ref":"LiveVue.SSR.ViteJS.html"},{"type":"function","title":"LiveVue.SSR.ViteJS.render/3","doc":"Callback implementation for  LiveVue.SSR.render/3 .","ref":"LiveVue.SSR.ViteJS.html#render/3"},{"type":"function","title":"LiveVue.SSR.ViteJS.vite_path/1","doc":"A handy utility returning path relative to Vite JS host.","ref":"LiveVue.SSR.ViteJS.html#vite_path/1"},{"type":"module","title":"LiveVue.Test","doc":"Helpers for testing LiveVue components and views.","ref":"LiveVue.Test.html"},{"type":"module","title":"Overview - LiveVue.Test","doc":"LiveVue testing differs from traditional Phoenix LiveView testing in how components\nare rendered and inspected:\n\n* In Phoenix LiveView testing, you use `Phoenix.LiveViewTest.render_component/2`\n  to get the final rendered HTML\n* In LiveVue testing, `render_component/2` returns an unrendered LiveVue root\n  element containing the Vue component's configuration\n\nThis module provides helpers to extract and inspect Vue component data from the\nLiveVue root element, including:\n\n* Component name and ID\n* Props passed to the component\n* Event handlers and their operations\n* Server-side rendering (SSR) status\n* Slot content\n* CSS classes","ref":"LiveVue.Test.html#module-overview"},{"type":"module","title":"Examples - LiveVue.Test","doc":"# Render a LiveVue component and inspect its properties\n    {:ok, view, _html} = live(conn, \"/\")\n    vue = LiveVue.Test.get_vue(view)\n\n    # Basic component info\n    assert vue.component == \"MyComponent\"\n    assert vue.props[\"title\"] == \"Hello\"\n\n    # Event handlers\n    assert vue.handlers[\"click\"] == JS.push(\"click\")\n\n    # SSR status and styling\n    assert vue.ssr == true\n    assert vue.class == \"my-custom-class\"","ref":"LiveVue.Test.html#module-examples"},{"type":"module","title":"Configuration - LiveVue.Test","doc":"","ref":"LiveVue.Test.html#module-configuration"},{"type":"module","title":"enable_props_diff - LiveVue.Test","doc":"When set to `false` in your config, LiveVue will always send full props and not send diffs.\nThis is useful for testing scenarios where you need to inspect the complete props state\nrather than just the changes.\n\n```elixir\n# config/test.exs\nconfig :live_vue,\n  enable_props_diff: false\n```\n\nWhen disabled, the `props` field returned by `get_vue/2` will always contain\nthe complete props state, making it easier to write comprehensive tests that verify the\nfull component state rather than just the incremental changes.","ref":"LiveVue.Test.html#module-enable_props_diff"},{"type":"function","title":"LiveVue.Test.get_vue/2","doc":"Extracts Vue component information from a LiveView or HTML string.\n\nWhen multiple Vue components are present, you can specify which one to extract using\neither the `:name` or `:id` option.\n\nReturns a map containing the component's configuration:\n  * `:component` - The Vue component name (from `v-component` attribute)\n  * `:id` - The unique component identifier (auto-generated or explicitly set)\n  * `:props` - The decoded props passed to the component\n  * `:handlers` - Map of event handlers (`v-on:*`) and their operations\n  * `:slots` - Base64 encoded slot content\n  * `:ssr` - Boolean indicating if server-side rendering was performed\n  * `:class` - CSS classes applied to the component root element","ref":"LiveVue.Test.html#get_vue/2"},{"type":"function","title":"Options - LiveVue.Test.get_vue/2","doc":"* `:name` - Find component by name (from `v-component` attribute)\n  * `:id` - Find component by ID","ref":"LiveVue.Test.html#get_vue/2-options"},{"type":"function","title":"Examples - LiveVue.Test.get_vue/2","doc":"# From a LiveView, get first Vue component\n    {:ok, view, _html} = live(conn, \"/\")\n    vue = LiveVue.Test.get_vue(view)\n\n    # Get specific component by name\n    vue = LiveVue.Test.get_vue(view, name: \"MyComponent\")\n\n    # Get specific component by ID\n    vue = LiveVue.Test.get_vue(view, id: \"my-component-1\")","ref":"LiveVue.Test.html#get_vue/2-examples"},{"type":"task","title":"mix live_vue.setup","doc":"Copies files from assets/copy of the live_vue dependency to phoenix project assets folder","ref":"Mix.Tasks.LiveVue.Setup.html"},{"type":"extras","title":"LiveVue","doc":"Vue inside Phoenix LiveView with seamless end-to-end reactivity.","ref":"readme.html"},{"type":"extras","title":"Features - LiveVue","doc":"-   ⚡ **End-To-End Reactivity** with LiveView\n-   🔋 **Server-Side Rendered** (SSR) Vue\n-   🐌 **Lazy-loading** Vue Components\n-   📦 **Efficient Props Diffing** - Only changed data is sent over WebSocket\n-   🪄 **Sigil** as an [Alternative LiveView DSL](#using-vue-sigil-to-inline-vue-components)\n-   🦄 **Tailwind** Support\n-   💀 **Dead View** Support\n-   🦥 **Slot Interoperability**\n-   🚀 **Amazing DX** with Vite","ref":"readme.html#features"},{"type":"extras","title":"Resources - LiveVue","doc":"-   [HexDocs](https://hexdocs.pm/live_vue)\n-   [HexPackage](https://hex.pm/packages/live_vue)\n-   [Phoenix LiveView](https://github.com/phoenixframework/phoenix_live_view)","ref":"readme.html#resources"},{"type":"extras","title":"Example - LiveVue","doc":"After installation, you can use Vue components in the same way as you'd use functional LiveView components. You can even handle Vue events with `JS` hooks! All the `phx-click`, `phx-change` attributes works inside Vue components as well.\n\n```html\n \nimport {ref} from \"vue\"\n\n// props are passed from LiveView\nconst props = defineProps<{count: number}>()\n\n// local state\nconst diff = ref(1)\n \n\n \n  Current count: {{ props.count }}\n   Diff:  \n   \n\n   \n    Increase counter by {{ diff }}\n   \n \n```\n\n```elixir\ndefmodule MyAppWeb.CounterLive do\n  use MyAppWeb, :live_view\n\n  def render(assigns) do\n    ~H\"\"\"\n    <.vue count={@count} v-component=\"Counter\" v-socket={@socket} />\n    \"\"\"\n  end\n\n  def mount(_params, _session, socket) do\n    {:ok, assign(socket, count: 0)}\n  end\n\n  def handle_event(\"inc\", %{\"diff\" => value}, socket) do\n    {:noreply, update(socket, :count, &(&1 + value))}\n  end\nend\n```","ref":"readme.html#example"},{"type":"extras","title":"Why? - LiveVue","doc":"Phoenix Live View makes it possible to create rich, interactive web apps without writing JS.\n\nBut once you'll need to do anything even slightly complex on the client-side, you'll end up writing lots of imperative, hard-to-maintain hooks.\n\nLiveVue allows to create hybrid apps, where part of the session state is on the server and part on the client.","ref":"readme.html#why"},{"type":"extras","title":"Reasons why you'd like to use LiveVue - LiveVue","doc":"-   Your hooks are starting to look like jQuery\n-   You have a complex local state\n-   You'd like to use a massive Vue ecosystem\n-   You want transitions, graphs etc.\n-   You simply like Vue 😉","ref":"readme.html#reasons-why-you-d-like-to-use-livevue"},{"type":"extras","title":"Installation - LiveVue","doc":"See [Installation](guides/installation.md).","ref":"readme.html#installation"},{"type":"extras","title":"Guides - LiveVue","doc":"","ref":"readme.html#guides"},{"type":"extras","title":"Getting Started - LiveVue","doc":"- [Getting Started](guides/getting_started.md) - Create your first Vue component with transitions","ref":"readme.html#getting-started"},{"type":"extras","title":"Core Usage - LiveVue","doc":"- [Basic Usage](guides/basic_usage.md) - Fundamental patterns, ~VUE sigil, and common examples\n - [Configuration](guides/configuration.md) - Advanced setup, SSR, and customization options","ref":"readme.html#core-usage"},{"type":"extras","title":"Reference - LiveVue","doc":"- [Component Reference](guides/component_reference.md) - Complete syntax documentation\n - [Client-Side API](guides/client_api.md) - Vue composables and utilities","ref":"readme.html#reference"},{"type":"extras","title":"Advanced Topics - LiveVue","doc":"- [Architecture](guides/architecture.md) - How LiveVue works under the hood\n - [Testing](guides/testing.md) - Testing Vue components in LiveView\n - [Deployment](guides/deployment.md) - Production deployment guide","ref":"readme.html#advanced-topics"},{"type":"extras","title":"Help & Troubleshooting - LiveVue","doc":"- [FAQ](guides/faq.md) - Common questions and comparisons\n - [Troubleshooting](guides/troubleshooting.md) - Debug common issues\n - [Comparison](guides/comparison.md) - LiveVue vs other solutions","ref":"readme.html#help-troubleshooting"},{"type":"extras","title":"Relation to LiveSvelte - LiveVue","doc":"This project is heavily inspired by ✨ [LiveSvelte](https://github.com/woutdp/live_svelte) ✨. Both projects try to solve the same problem. LiveVue was started as a fork of LiveSvelte with adjusted ESbuild settings, and evolved to use Vite and a slightly different syntax. I strongly believe more options are always better, and since I love Vue and it's ecosystem I've decided to give it a go 😉\n\nYou can read more about differences between Vue and Svelte [in FAQ](#differences-from-livesvelte).","ref":"readme.html#relation-to-livesvelte"},{"type":"extras","title":"LiveVue Development - LiveVue","doc":"","ref":"readme.html#livevue-development"},{"type":"extras","title":"Local Setup - LiveVue","doc":"Ensure you have node installed. Clone the repo and run `mix setup`. You can then run `mix assets.watch` to start a watcher for the assets.\n\n#### Example Project\n\nYou can use `/example_project` as a way to test `live_vue` locally.\n\n- Clone this repo\n- Go to example_project dir\n- Run `mix setup`\n- Run `mix phx.server` to start the server\n- Open [http://localhost:4000](http://localhost:4000) in your browser","ref":"readme.html#local-setup"},{"type":"extras","title":"Building Static Files - LiveVue","doc":"LiveVue is written in TypeScript. To see the changes you made, you have to recompile the assets. You can do it with:\n\n```bash\nmix assets.build\n```\n\nOr run the watcher:\n\n```bash\nmix assets.watch\n```","ref":"readme.html#building-static-files"},{"type":"extras","title":"Releasing - LiveVue","doc":"Release is done with `expublish` package.\n\n-   Write version changelog in untracked `RELEASE.md` file\n-   Update version in `INSTALLATION.md`\n\nRun\n\n```bash\ngit add INSTALLATION.md\ngit commit -m \"INSTALLATION version bump\"\n\n# to ensure everything works fine\nmix expublish.minor --dry-run --allow-untracked --branch=main\n\n# to publish everything\nmix do assets.build, expublish.minor --allow-untracked --branch=main\n```","ref":"readme.html#releasing"},{"type":"extras","title":"Roadmap 🎯 - LiveVue","doc":"- [x] `useLiveEvent` - an utility automatically attaching & detaching [`handleEvent`](https://hexdocs.pm/phoenix_live_view/js-interop.html#client-hooks-via-phx-hook)\n- [x] ~~optimize payload - send only [`json_patch`](https://hexdocs.pm/jsonpatch/readme.html) diffs of updated props~~ ✅ **Done!**\n- [ ] shared props automatically included in all components\n- [ ] VS code extension highlighting `~VUE` sigil\n- [ ] try to use [Igniter](https://hexdocs.pm/igniter/) as a way of installing LiveVue in a project\n- [ ] `usePushEvent` - an utility similar to [`useFetch`](https://vueuse.org/core/useFetch/) making it easy to get data from `&handle_event/3 -> {:reply, data, socket}` responses\n- [ ] `useLiveForm` - an utility to efforlessly use Ecto changesets & server-side validation, similar to HEEX\n- [ ] Add support for Phoenix streams as props","ref":"readme.html#roadmap"},{"type":"extras","title":"Credits - LiveVue","doc":"[LiveSvelte](https://github.com/woutdp/live_svelte)","ref":"readme.html#credits"},{"type":"extras","title":"Star History - LiveVue","doc":"[![Star History Chart](https://api.star-history.com/svg?repos=Valian/live_vue&type=Date)](https://star-history.com/#Valian/live_vue&Date)","ref":"readme.html#star-history"},{"type":"extras","title":"Installation","doc":"# Installation\n\nLiveVue replaces `esbuild` with [Vite](https://vitejs.dev/) for both client side code and SSR to achieve an amazing development experience.","ref":"installation.html"},{"type":"extras","title":"Why Vite? - Installation","doc":"- Vite provides a best-in-class Hot-Reload functionality and offers [many benefits](https://vitejs.dev/guide/why#why-vite)\n- `esbuild` package doesn't support plugins, so we would need to setup a custom build process anyway\n- In production, we'll use [elixir-nodejs](https://github.com/revelrylabs/elixir-nodejs) for SSR\n\nIf you don't need SSR, you can easily disable it with one line of configuration.","ref":"installation.html#why-vite"},{"type":"extras","title":"Prerequisites - Installation","doc":"- Node.js installed (version 19 or later recommended)\n- Phoenix 1.7+ project\n- Elixir 1.13+","ref":"installation.html#prerequisites"},{"type":"extras","title":"Setup Steps - Installation","doc":"> #### Installation with Igniter {: .tip}\n>\n> I'm aware that installation process takes a moment to complete, but it's worth it! 🫣\n> There's also ongoing work to create an igniter installer for LiveVue, should be ready in the next version 🤞\n\n1. Add LiveVue to your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:live_vue, \"~> 0.6\"}\n  ]\nend\n```\n\n2. Configure environments:\n\n```elixir\n# in config/dev.exs\nconfig :live_vue,\n  vite_host: \"http://localhost:5173\",\n  ssr_module: LiveVue.SSR.ViteJS,\n  # if you want to disable SSR by default, set this to false\n  ssr: true\n\n# in config/prod.exs\nconfig :live_vue,\n  ssr_module: LiveVue.SSR.NodeJS,\n  ssr: true\n```\n\n3. Add LiveVue to your `html_helpers` in `lib/my_app_web.ex`:\n\n```elixir\ndefp html_helpers do\n  quote do\n    # ... existing code ...\n\n    # Add support for Vue components\n    use LiveVue\n\n    # Generate component for each vue file, so you can use <.ComponentName> syntax\n    # instead of <.vue v-component=\"ComponentName\">\n    use LiveVue.Components, vue_root: [\"./assets/vue\", \"./lib/my_app_web\"]\n  end\nend\n```\n\n4. Run the setup command to generate required files:\n\n```bash\nmix deps.get\nmix live_vue.setup\ncd assets && npm install\n```\n\nThe `live_vue.setup` command will create:\n- `package.json` with required dependencies\n- Vite, TypeScript, and PostCSS configs\n- Server entrypoint for SSR\n- Vue entrypoint files\n\n5. Update your JavaScript configuration:\n\n```javascript\n// app.js\nimport topbar from \"topbar\" // instead of ../vendor/topbar\nimport {getHooks} from \"live_vue\"\nimport liveVueApp from \"../vue\"\n\n// remember to import your css here\nimport \"../css/app.css\"\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n    // ... existing options ...\n    hooks: getHooks(liveVueApp),\n})\n```\n\n6. Configure Tailwind to include Vue files:\n\n```javascript\n// tailwind.config.js\nmodule.exports = {\n    content: [\n        // ... existing patterns\n        \"./vue/**/*.vue\",\n        \"../lib/**/*.vue\",\n    ],\n}\n```\n\n7. Update root.html.heex for Vite:\n\n```heex\n \n   \n   \n   \n \n```\n\n8. Remove esbuild and tailwind packages from your dependencies in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    # Remove these lines:\n    # {:esbuild, \"~> 0.8\", runtime: Mix.env() == :dev},\n    # {:tailwind, \"~> 0.2\", runtime: Mix.env() == :dev},\n\n    # ... other dependencies ...\n  ]\nend\n```\n\n9. Update your `mix.exs` aliases:\n\n```elixir\ndefp aliases do\n  [\n    setup: [\"deps.get\", \"assets.setup\", \"assets.build\"],\n    \"assets.setup\": [\"cmd --cd assets npm install\"],\n    \"assets.build\": [\n      \"cmd --cd assets npm run build\",\n      \"cmd --cd assets npm run build-server\"\n    ],\n    \"assets.deploy\": [\n      \"cmd --cd assets npm run build\",\n      \"cmd --cd assets npm run build-server\",\n      \"phx.digest\"\n    ]\n  ]\nend\n```\n\n10. Remove esbuild and tailwind config from `config/config.exs`\n\n11. Configure watchers in `config/dev.exs`:\n\n```elixir\nconfig :my_app, MyAppWeb.Endpoint,\n  watchers: [\n    npm: [\"--silent\", \"run\", \"dev\", cd: Path.expand(\"../assets\", __DIR__)]\n  ]\n```\n\n12. Setup SSR for production in `application.ex`:\n\n```elixir\nchildren = [\n  {NodeJS.Supervisor, [path: LiveVue.SSR.NodeJS.server_path(), pool_size: 4]},\n  # ... other children\n]\n```\n\n13. Confirm everything is working by rendering an example Vue component in one of your LiveViews:\n\n```elixir\n~H\"\"\"\n<.vue\n  count={@count}\n  v-component=\"Counter\"\n  v-socket={@socket}\n  v-on:inc={JS.push(\"inc\")}\n/>\n\"\"\"\n```\n\n14. (Optional) Enable [stateful hot reload](https://twitter.com/jskalc/status/1788308446007132509) for LiveViews:\n\nThis provides a superior development experience by preserving the LiveView's state (e.g., form data, temporary assigns) while Vite instantly updates the Vue component's code on the client. You get the best of both worlds: state persistence from the server and immediate UI updates from the client.\n\n```elixir\n# config/dev.exs\nconfig :my_app, MyAppWeb.Endpoint,\n  live_reload: [\n    notify: [\n      live_view: [\n        ~r\"lib/my_app_web/core_components.ex$\",\n        ~r\"lib/my_app_web/(live|components)/.*(ex|heex)$\"\n      ]\n    ],\n    patterns: [\n      ~r\"priv/static/(?!uploads/).*(js|css|png|jpeg|jpg|gif|svg)$\",\n      ~r\"lib/my_app_web/controllers/.*(ex|heex)$\"\n    ]\n  ]\n```","ref":"installation.html#setup-steps"},{"type":"extras","title":"Manually Adjusting package.json - Installation","doc":"If you need to set up your `package.json` manually instead of using `mix live_vue.setup`, follow these steps:\n\n1. Install the required packages:\n\n```bash\ncd assets\n\n# vite\nnpm install -D vite @vitejs/plugin-vue\n\n# tailwind\nnpm install -D tailwindcss autoprefixer postcss @tailwindcss/forms\n\n# typescript\nnpm install -D typescript vue-tsc\n\n# runtime dependencies\nnpm install --save vue topbar ../deps/live_vue ../deps/phoenix ../deps/phoenix_html ../deps/phoenix_live_view\n\n# remove topbar from vendor, since we'll use it from node_modules\nrm vendor/topbar.js\n```\n\n2. Add these scripts to your `package.json`:\n\n```json\n{\n    \"private\": true,\n    \"type\": \"module\",\n    \"scripts\": {\n        \"dev\": \"vite --host -l warn\",\n        \"build\": \"vue-tsc && vite build\",\n        \"build-server\": \"vue-tsc && vite build --ssr js/server.js --out-dir ../priv/vue --minify esbuild --ssrManifest && echo '{\\\"type\\\": \\\"module\\\" } ' > ../priv/vue/package.json\"\n    }\n}\n```","ref":"installation.html#manually-adjusting-package-json"},{"type":"extras","title":"Troubleshooting - Installation","doc":"","ref":"installation.html#troubleshooting"},{"type":"extras","title":"TypeScript Compatibility Issues - Installation","doc":"There's a [known bug](https://github.com/vuejs/language-tools/issues/5018) where recent versions of TypeScript are not compatible with the latest version of `vue-tsc`. If you encounter an error like:\n\n```bash\nnpm install typescript@5.5.4 vue-tsc@2.10.0\n```\n\nYou'll need to downgrade TypeScript and vue-tsc to specific versions:\n\n```bash\nnpm install typescript@5.5.4 vue-tsc@2.10.0\n```\n\nThis issue has been documented in [LiveVue issue #43](https://github.com/Valian/live_vue/issues/43#issuecomment-2501152160).","ref":"installation.html#typescript-compatibility-issues"},{"type":"extras","title":"Protocol.UndefinedError with Custom Structs - Installation","doc":"If you encounter a `Protocol.UndefinedError` mentioning `LiveVue.Encoder` when passing custom structs as props, you need to implement the encoder protocol:\n\n```elixir\n# Add this to your struct definitions\ndefmodule User do\n  @derive LiveVue.Encoder\n  defstruct [:name, :email, :age]\nend\n```\n\nThis is a safety feature to prevent accidental exposure of sensitive data. For more details, see [Component Reference](component_reference.html#custom-structs-with-livevue-encoder).","ref":"installation.html#protocol-undefinederror-with-custom-structs"},{"type":"extras","title":"Next Steps - Installation","doc":"Now that you have LiveVue installed, check out our [Getting Started Guide](getting_started.html) to create your first Vue component!","ref":"installation.html#next-steps"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\nNow that you have LiveVue installed, let's create your first Vue component and integrate it with LiveView.","ref":"getting_started.html"},{"type":"extras","title":"Creating Your First Component - Getting Started","doc":"Let's create a simple counter component that demonstrates the reactivity between Vue and LiveView.\n\n1. Create `assets/vue/Counter.vue`:\n\n```html\n \nimport {ref} from \"vue\"\n\n// props are passed from LiveView\nconst props = defineProps<{count: number}>()\n\n// local state\nconst diff = ref(1)\n \n\n \n  Current count: {{ props.count }}\n   Diff:  \n   \n\n   \n    Increase counter by {{ diff }}\n   \n \n```\n\n2. Create a LiveView to handle the counter state (`lib/my_app_web/live/counter_live.ex`):\n\n```elixir\ndefmodule MyAppWeb.CounterLive do\n  use MyAppWeb, :live_view\n\n  def render(assigns) do\n    ~H\"\"\"\n    <.vue count={@count} v-component=\"Counter\" v-socket={@socket} />\n    \"\"\"\n  end\n\n  def mount(_params, _session, socket) do\n    {:ok, assign(socket, count: 0)}\n  end\n\n  def handle_event(\"inc\", %{\"diff\" => value}, socket) do\n    {:noreply, update(socket, :count, &(&1 + value))}\n  end\nend\n```\n\n3. Add the route in your `router.ex`:\n\n```elixir\nlive \"/counter\", CounterLive\n```\n\nStart server and visit `http://localhost:4000/counter` to see your counter in action!\nIf it's not working correctly, see [Troubleshooting](troubleshooting.html).","ref":"getting_started.html#creating-your-first-component"},{"type":"extras","title":"Adding Smooth Transitions - Getting Started","doc":"One of Vue's strengths is its built-in transition system. Let's enhance our counter with smooth animations and nice tailwind styling:\n\n1. Create `assets/vue/AnimatedCounter.vue`:\n\n```html\n \nimport {ref} from \"vue\"\n\nconst props = defineProps<{count: number}>()\nconst diff = ref(1)\n \n\n \n   \n     \n       \n         \n          {{ props.count }}\n         \n       \n     \n\n     \n       Diff: \n       \n       {{ diff }} \n     \n\n     \n      Increase by {{ diff }}\n     \n   \n \n\n \n.count-enter-active,\n.count-leave-active {\n  transition: all 0.3s ease;\n}\n\n.count-enter-from {\n  opacity: 0;\n  transform: scale(0.8) translateY(-10px);\n}\n\n.count-leave-to {\n  opacity: 0;\n  transform: scale(1.2) translateY(10px);\n}\n \n```\n\n2. Update your LiveView to use the animated version:\n\n```elixir\ndef render(assigns) do\n  ~H\"\"\"\n   \n     Animated Counter \n    <.vue count={@count} v-component=\"AnimatedCounter\" v-socket={@socket} />\n   \n  \"\"\"\nend\n```\n\nNow your counter will smoothly animate when the value changes! This showcases how Vue's transition system can add polish to your LiveView apps without any server-side complexity.","ref":"getting_started.html#adding-smooth-transitions"},{"type":"extras","title":"Key Concepts - Getting Started","doc":"This example demonstrates several key LiveVue features:\n\n- **Props Flow**: LiveView sends the `count` value to Vue as a prop\n- **Event Handling**: Vue emits an `inc` event with `phx-click` and `phx-value-diff` attributes\n- **State Management**: LiveView maintains the source of truth (the counter value)\n- **Local UI State**: Vue maintains the slider value locally without server involvement\n- **Transitions**: Vue handles smooth animations purely on the client side\n\nBasic diagram of the flow:\n\n![LiveVue flow](./images/lifecycle.png)\n\nIf you want to understand how it works in depth, see [Architecture](architecture.html).","ref":"getting_started.html#key-concepts"},{"type":"extras","title":"Working with Custom Structs - Getting Started","doc":"When you start passing more complex data structures as props, you'll need to implement the `LiveVue.Encoder` protocol:\n\n```elixir\n# For any custom structs you want to pass as props\ndefmodule User do\n  @derive LiveVue.Encoder\n  defstruct [:name, :email, :age]\nend\n\n# Use in your LiveView\ndef render(assigns) do\n  ~H\"\"\"\n  <.vue user={@current_user} v-component=\"UserProfile\" v-socket={@socket} />\n  \"\"\"\nend\n```\n\nThis protocol ensures that:\n- Only specified fields are sent to the client\n- Sensitive data is protected from accidental exposure\n- Props can be efficiently diffed for optimal performance\n\nFor more details, see [Component Reference](component_reference.html#custom-structs-with-livevue-encoder).\n\n\n> #### Good to know {: .info}\n>\n> - Install the [Vue DevTools browser extension](https://devtools.vuejs.org/getting-started/installation) for debugging\n> - In development, LiveVue enables Hot Module Replacement for instant component updates\n> - Structure your app with LiveView managing application state and Vue handling UI interactions\n> - For complex UIs with lots of local state, prefer Vue components over LiveView hooks","ref":"getting_started.html#working-with-custom-structs"},{"type":"extras","title":"Next Steps - Getting Started","doc":"Now that you have your first component working, explore:\n- [Basic Usage](basic_usage.html) for more patterns and the ~VUE sigil\n- [Component Reference](component_reference.html) for complete syntax documentation\n- [FAQ](faq.html) for common questions and troubleshooting\n- [Troubleshooting](troubleshooting.html) for common issues","ref":"getting_started.html#next-steps"},{"type":"extras","title":"Basic Usage","doc":"# Basic Usage\n\nThis guide covers the fundamental patterns for using Vue components within LiveView.","ref":"basic_usage.html"},{"type":"extras","title":"Component Organization - Basic Usage","doc":"By default, Vue components should be placed in either:\n- `assets/vue` directory\n- Colocated with your LiveView files in `lib/my_app_web`\n\nFor advanced component organization and custom resolution patterns, see [Configuration](configuration.html#component-organization).","ref":"basic_usage.html#component-organization"},{"type":"extras","title":"Rendering Components - Basic Usage","doc":"","ref":"basic_usage.html#rendering-components"},{"type":"extras","title":"Basic Syntax - Basic Usage","doc":"To render a Vue component from HEEX, use the `<.vue>` function:\n\n```elixir\n<.vue\n  count={@count}\n  v-component=\"Counter\"\n  v-socket={@socket}\n  v-on:inc={JS.push(\"inc\")}\n/>\n```","ref":"basic_usage.html#basic-syntax"},{"type":"extras","title":"Required Attributes - Basic Usage","doc":"| Attribute    | Example                | Required        | Description                                    |\n|--------------|------------------------|-----------------|------------------------------------------------|\n| v-component  | `v-component=\"Counter\"`| Yes            | Component name or path relative to vue_root    |\n| v-socket     | `v-socket={@socket}`   | Yes in LiveView| Required for SSR and reactivity               |","ref":"basic_usage.html#required-attributes"},{"type":"extras","title":"Optional Attributes - Basic Usage","doc":"| Attribute    | Example              | Description                                    |\n|--------------|----------------------|------------------------------------------------|\n| v-ssr        | `v-ssr={true}`      | Override default SSR setting                   |\n| v-on:event   | `v-on:inc={JS.push(\"inc\")}` | Handle Vue component events           |\n| prop={@value}| `count={@count}`     | Pass props to the component                   |","ref":"basic_usage.html#optional-attributes"},{"type":"extras","title":"Component Shortcut - Basic Usage","doc":"Instead of writing `<.vue v-component=\"Counter\">`, you can use the shortcut syntax:\n\n```elixir\n<.Counter count={@count} v-socket={@socket} />\n```\n\nFunction names are generated based on `.vue` file names. For files with identical names, use the full path:\n\n```elixir\n<.vue v-component=\"helpers/nested/Modal\" />\n```","ref":"basic_usage.html#component-shortcut"},{"type":"extras","title":"Passing Props - Basic Usage","doc":"Props can be passed in three equivalent ways:\n\n```elixir\n# Individual props\n<.vue count={@count} max={123} v-component=\"Counter\" v-socket={@socket} />\n\n# Map spread\n<.vue v-component=\"Counter\" v-socket={@socket} {@props} />\n\n# Using shortcut - you don't have to specify v-component\n<.Counter count={@count} max={123} v-socket={@socket} />\n```","ref":"basic_usage.html#passing-props"},{"type":"extras","title":"Custom Structs as Props - Basic Usage","doc":"When passing custom structs as props, you must implement the `LiveVue.Encoder` protocol:\n\n```elixir\ndefmodule User do\n  @derive LiveVue.Encoder\n  defstruct [:name, :email, :age]\nend\n\n# Use in your LiveView\ndef render(assigns) do\n  ~H\"\"\"\n  <.vue\n    user={@current_user}\n    v-component=\"UserProfile\"\n    v-socket={@socket}\n  />\n  \"\"\"\nend\n```\n\nThe encoder protocol ensures that only specified fields are sent to the client, sensitive data is protected, and props can be efficiently diffed for updates.\n\nFor complete implementation details including field selection and custom implementations, see [Component Reference](component_reference.html#custom-structs-with-livevue-encoder).\n\n> #### Protocol.UndefinedError {: .warning}\n>\n> If you get a `Protocol.UndefinedError` when passing structs as props, it means you need to implement the `LiveVue.Encoder` protocol for that struct. This is a safety feature to prevent accidental exposure of sensitive data.","ref":"basic_usage.html#custom-structs-as-props"},{"type":"extras","title":"Handling Events - Basic Usage","doc":"","ref":"basic_usage.html#handling-events"},{"type":"extras","title":"Phoenix Events - Basic Usage","doc":"All standard Phoenix event handlers work inside Vue components:\n- `phx-click`\n- `phx-change`\n- `phx-submit`\n- etc.\n\nThey will be pushed directly to LiveView, exactly as happens with `HEEX` components.","ref":"basic_usage.html#phoenix-events"},{"type":"extras","title":"Programmatic access to hook instance - Basic Usage","doc":"There are two ways to access the Phoenix LiveView hook instance from your Vue components:\n\n1.  **`useLiveVue()` Composable (in ` `):**\n\n    Use the `useLiveVue()` composable when you need to access the hook instance for logic within your ` ` block. It's ideal for pushing events programmatically.\n\n    ```html\n     \n    import { useLiveVue } from \"live_vue\"\n    import { ref } from \"vue\"\n\n    const live = useLiveVue()\n    const name = ref(\"\")\n\n    function save() {\n      live.pushEvent(\"save\", { name: name.value })\n    }\n     \n    ```\n\n    To listen for events from the server, the easiest way is to use the `useLiveEvent` composable, which will automatically handle cleanup for you.\n\n    ```html\n     \n    import { useLiveEvent } from \"live_vue\"\n\n    // Example: listening for a server event\n    useLiveEvent(\"response\", (payload) => { console.log(payload) })\n     \n    ```\n\n2.  **`$live` Property (in ` `):**\n\n    For convenience, the hook instance is also available directly in your template as the `$live` property. This is the preferred method for simple, one-off event pushes directly from the template, as it avoids the need to import and call `useLiveVue()`.\n\n    ```html\n     \n       \n        Click me\n       \n     \n    ```\n\nThe `live` object provides all methods from [Phoenix.LiveView JS Interop](https://hexdocs.pm/phoenix_live_view/js-interop.html#client-hooks-via-phx-hook). For a complete API reference, see [Client-Side API](client_api.html).","ref":"basic_usage.html#programmatic-access-to-hook-instance"},{"type":"extras","title":"LiveView Navigation - Basic Usage","doc":"For navigation, LiveVue provides a built-in `Link` component that makes using `live_patch` and `live_redirect` as easy as using a standard ` ` tag.\n\n```html\n \nimport { Link } from \"live_vue\"\n \n\n \n   \n    <!-- Behaves like a normal link -->\n     About \n\n    <!-- Performs a `live_patch` -->\n     Sort by Name \n\n    <!-- Performs a `live_redirect` -->\n     Dashboard \n   \n \n```\n\nFor a complete API reference, see [Client-Side API](client_api.html#link).","ref":"basic_usage.html#liveview-navigation"},{"type":"extras","title":"Vue Events - Basic Usage","doc":"If you want to create reusable Vue components where you'd like to define what happens when Vue emits an event, you can use the `v-on:` syntax with `JS` [module helpers](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.JS.html#module-client-utility-commands).\n\n```elixir\n<.vue\n  v-on:submit={JS.push(\"submit\")}\n  v-on:close={JS.hide()}\n  v-component=\"Form\"\n  v-socket={@socket}\n/>\n```\n\nSpecial case: When using `JS.push()` without a value, it automatically uses the emit payload:\n```elixir\n# In Vue\nemit('inc', {value: 5})\n\n# In LiveView\n<.vue v-on:inc={JS.push(\"inc\")} />\n# Equivalent to: JS.push(\"inc\", value: 5)\n```","ref":"basic_usage.html#vue-events"},{"type":"extras","title":"Slots Support - Basic Usage","doc":"Vue components can receive slots from LiveView templates:\n\n```elixir\n<.Card title=\"Example Card\" v-socket={@socket}>\n   This is the default slot content! \n   Phoenix components work too: <.icon name=\"hero-info\" /> \n\n  <:footer>\n    This is a named slot\n  </:footer>\n</.Card>\n```\n\n```html\n \n   \n    <!-- Default slot -->\n      \n\n    <!-- Named slot -->\n      \n   \n \n```\n\nImportant notes about slots:\n- Each slot is wrapped in a div (technical limitation)\n- You can use HEEX components inside slots 🥳\n- Slots stay reactive and update when their content changes\n\n\n> #### Hooks inside slots are not supported {: .warning}\n>\n> Slots are rendered server-side and then sent to the client as a raw HTML.\n> It happens outside of the LiveView lifecycle, so hooks inside slots are not supported.\n>\n> As a consequence, since `.vue` components rely on hooks, it's not possible to nest `.vue` components inside other `.vue` components.","ref":"basic_usage.html#slots-support"},{"type":"extras","title":"Dead Views vs Live Views - Basic Usage","doc":"Components can be used in both contexts:\n- Live Views: Full reactivity with WebSocket updates\n- Dead Views: Static rendering, no reactivity\n  - `v-socket={@socket}` not required\n  - SSR still works for initial render","ref":"basic_usage.html#dead-views-vs-live-views"},{"type":"extras","title":"Using ~VUE Sigil - Basic Usage","doc":"The `~VUE` sigil provides an alternative to the standard LiveView DSL, allowing you to write Vue components directly in your LiveView:\n\n> #### Deprecation Notice {: .warning}\n>\n> The `~V` sigil is deprecated in favor of `~VUE`. It will be removed in future versions.\n\n```elixir\ndefmodule MyAppWeb.CounterLive do\n  use MyAppWeb, :live_view\n\n  def render(assigns) do\n    ~VUE\"\"\"\n     \n    import {ref} from \"vue\"\n    const props = defineProps<{count: number}>()\n    const diff = ref(1)\n     \n\n     \n      Current count: {{ props.count }}\n       Diff:  \n       \n\n       \n        Increase counter by {{ diff }}\n       \n     \n    \"\"\"\n  end\n\n  def mount(_params, _session, socket) do\n    {:ok, assign(socket, count: 0)}\n  end\n\n  def handle_event(\"inc\", %{\"diff\" => diff}, socket) do\n    {:noreply, update(socket, :count, &(&1 + String.to_integer(diff)))}\n  end\nend\n```\n\nThe `~VUE` sigil is a powerful macro that compiles the string content into a full-fledged Vue component at compile time. It automatically passes all of the LiveView's `assigns` as props to the component, making it easy to create reactive components.\n\n**When to use the `~VUE` sigil:**\n\n*   **Prototyping:** Quickly build and iterate on new components without creating new files.\n*   **Single-use components:** Ideal for components that are tightly coupled to a specific LiveView and won't be reused.\n*   **Co-location:** Keep server-side and client-side logic for a piece of functionality within a single file.\n\n**When to use `.vue` files instead:**\n\n*   **Reusability:** When you need to use the same component in multiple LiveViews.\n*   **Large components:** For complex components, a dedicated file improves organization and editor support.\n*   **Collaboration:** Separate files are often easier for teams to work on simultaneously.","ref":"basic_usage.html#using-vue-sigil"},{"type":"extras","title":"Next Steps - Basic Usage","doc":"Now that you understand the basics, you might want to explore:\n\n- [Component Reference](component_reference.html) for complete syntax documentation\n- [Configuration](configuration.html) for advanced setup and customization options\n- [Client-Side API](client_api.html) for detailed API reference and advanced patterns\n- [FAQ](faq.html) for common questions and troubleshooting","ref":"basic_usage.html#next-steps"},{"type":"extras","title":"Configuration","doc":"# Configuration\n\nThis guide covers all configuration options available in LiveVue, from basic setup to advanced customization.","ref":"configuration.html"},{"type":"extras","title":"Application Configuration - Configuration","doc":"LiveVue configuration is managed in your `config/config.exs` file:\n\n```elixir\nimport Config\n\nconfig :live_vue,\n  # SSR module selection\n  # For development: LiveVue.SSR.ViteJS\n  # For production: LiveVue.SSR.NodeJS\n  ssr_module: nil,\n\n  # Default SSR behavior\n  # Can be overridden per-component with v-ssr={true|false}\n  ssr: true,\n\n  # Vite development server URL\n  # Typically http://localhost:5173 in development\n  vite_host: nil,\n\n  # SSR server bundle path (relative to priv directory)\n  # Created by Vite \"build-server\" command\n  ssr_filepath: \"./vue/server.js\",\n\n  # Testing configuration\n  # When false, we will always update full props and not send diffs\n  # Useful for testing scenarios where you need complete props state\n  enable_props_diff: true\n```","ref":"configuration.html#application-configuration"},{"type":"extras","title":"Environment-Specific Configuration - Configuration","doc":"#### Recommended development Configuration\n\n```elixir\n# config/dev.exs\nconfig :live_vue,\n  ssr_module: LiveVue.SSR.ViteJS,\n  vite_host: \"http://localhost:5173\",\n  ssr: true\n```\n\n#### Recommended Production Configuration\n\n```elixir\n# config/prod.exs\nconfig :live_vue,\n  ssr_module: LiveVue.SSR.NodeJS,\n  ssr: true\n\n# or if you don't want to use SSR\nconfig :live_vue,\n  ssr_module: nil,\n  ssr: false\n```","ref":"configuration.html#environment-specific-configuration"},{"type":"extras","title":"Vue Application Setup - Configuration","doc":"Configure your Vue application in `assets/vue/index.js`. You should use createLiveVue to provide two required functions:\n\n| Option | Type | Description |\n|--------|------|-------------|\n| `resolve` | `(name: string) => Component \\| Promise ` | Component resolution function |\n| `setup` | `(context: SetupContext) => VueApp` | Vue app setup function |\n\nInstallation step provides a reasonable implementation of createLiveVue that you can use as a starting point.","ref":"configuration.html#vue-application-setup"},{"type":"extras","title":"Basic Configuration - Configuration","doc":"```javascript\nimport \"vite/modulepreload-polyfill\"\nimport { h } from \"vue\"\nimport { createLiveVue, findComponent } from \"live_vue\"\n\nexport default createLiveVue({\n  // Component resolution - adjust this to your needs\n  // Eg. You might want to import some components directly from node_modules\n  // or lazy load components\n  resolve: name => {\n    const components = {\n      ...import.meta.glob(\"./**/*.vue\", { eager: true }),\n      ...import.meta.glob(\"../../lib/**/*.vue\", { eager: true }),\n    }\n\n    // findComponent resolves the component based on suffix.\n    // Equivalent to this snippet + some error handling:\n    // for (const [key, value] of Object.entries(components)) {\n    //   if (key.endsWith(`${name}.vue`) || key.endsWith(`${name}/index.vue`)) {\n    //     return value\n    //   }\n    // }\n    return findComponent(components, name)\n  },\n\n  // Vue app setup\n  setup: ({ createApp, component, props, slots, plugin, el }) => {\n    const app = createApp({ render: () => h(component, props, slots) })\n    app.use(plugin)\n    app.mount(el)\n    return app\n  },\n})\n```","ref":"configuration.html#basic-configuration"},{"type":"extras","title":"SetupContext - Configuration","doc":"SetupContext is an object that is passed to the setup function.\n\n| Property | Type | Description |\n|----------|------|-------------|\n| `createApp` | `Function` | Vue's createApp or createSSRApp |\n| `component` | `Component` | The Vue component to render |\n| `props` | `object` | Props passed from LiveView |\n| `slots` | `object` | Slots passed from LiveView |\n| `plugin` | `Plugin` | LiveVue plugin (required) |\n| `el` | `HTMLElement` | Mount target element |\n| `ssr` | `boolean` | Whether this is SSR context |","ref":"configuration.html#setupcontext"},{"type":"extras","title":"Component Resolution Options - Configuration","doc":"#### Eager Loading (Default)\nAll components are bundled with the main application:\n\n```javascript\nconst components = {\n  ...import.meta.glob(\"./**/*.vue\", { eager: true }),\n}\n```\n\n#### Lazy Loading\nIf you want to lazy load components, you can use the `import` function:\n\n```javascript\nconst components = {\n  Counter: () => import(\"./Counter.vue\"),\n  Modal: () => import(\"./Modal.vue\")\n}\n\n// Or using Vite's glob import\n// useful if we colocate Vue components with LiveView components and want to put each of them into a separate chunk\n// all shared components imported by top-level components will be included as well.\nconst components = import.meta.glob(\n  \"../../lib/**/*.vue\",\n  { eager: false, import: 'default' }\n)\n```\n\n#### Custom Resolution Logic\n\n```javascript\nresolve: name => {\n  // Custom component mapping and lazy loading\n  const componentMap = {\n    'MyCounter': () => import('./components/Counter.vue'),\n    'admin/Dashboard': () => import('./admin/Dashboard.vue')\n  }\n\n  return componentMap[name]\n}\n```","ref":"configuration.html#component-resolution-options"},{"type":"extras","title":"Vue App Customization - Configuration","doc":"Add plugins, stores, and other Vue features:\n\n```javascript\nimport { createPinia } from \"pinia\"\nimport { createI18n } from \"vue-i18n\"\n\nexport default createLiveVue({\n  setup: ({ createApp, component, props, slots, plugin, el, ssr }) => {\n    const app = createApp({ render: () => h(component, props, slots) })\n\n    // LiveVue plugin (required)\n    app.use(plugin)\n\n    // Add your plugins\n    const pinia = createPinia()\n    app.use(pinia)\n\n    const i18n = createI18n({\n      locale: 'en',\n      messages: { /* your translations */ }\n    })\n    app.use(i18n)\n\n    // SSR-specific setup\n    if (ssr) {\n      // Server-side specific initialization\n    }\n\n    app.mount(el)\n    return app\n  }\n})\n```","ref":"configuration.html#vue-app-customization"},{"type":"extras","title":"Component Organization - Configuration","doc":"","ref":"configuration.html#component-organization"},{"type":"extras","title":"Directory Structure - Configuration","doc":"By default, Vue components are resolved from:\n- `assets/vue/` - Main Vue components directory\n- `lib/my_app_web/` - Colocated with LiveView files","ref":"configuration.html#directory-structure"},{"type":"extras","title":"Custom Vue Root Directories - Configuration","doc":"Configure component discovery paths in your LiveView module:\n\n```elixir\n# lib/my_app_web.ex\ndefmodule MyAppWeb do\n  def html_helpers do\n    quote do\n      use LiveVue.Components, vue_root: [\n        \"./assets/vue\",\n        \"./lib/my_app_web\",\n        \"./lib/my_app_web/components\"\n      ]\n    end\n  end\nend\n```\n\nThis generates shortcut functions for your components:\n\n```elixir\n# Instead of\n<.vue v-component=\"Counter\" v-socket={@socket} />\n\n# You can use\n<.Counter v-socket={@socket} />\n```","ref":"configuration.html#custom-vue-root-directories"},{"type":"extras","title":"Component Naming Conventions - Configuration","doc":"Components are resolved by name or path suffix:\n- `Counter.vue` → accessible as `\"Counter\"`\n- `path/to/Component.vue` → accessible as `\"path/to/Component\"` or `\"Component\"`\n- `path/to/component/index.vue` → accessible as `\"path/to/component\"` or `\"component\"`","ref":"configuration.html#component-naming-conventions"},{"type":"extras","title":"Server-Side Rendering (SSR) - Configuration","doc":"LiveVue provides flexible SSR options that work great in both development and production environments.","ref":"configuration.html#server-side-rendering-ssr"},{"type":"extras","title":"SSR Modules - Configuration","doc":"LiveVue offers two SSR strategies depending on your environment:\n\n#### ViteJS (Development)\nPerfect for development with hot module replacement:\n\n```elixir\n# config/dev.exs\nconfig :live_vue,\n  ssr_module: LiveVue.SSR.ViteJS,\n  vite_host: \"http://localhost:5173\"\n```\n\nUses Vite's `ssrLoadModule` for efficient development compilation with instant updates.\n\n#### NodeJS (Production)\nOptimized for production with an in-memory server bundle:\n\n```elixir\n# config/prod.exs\nconfig :live_vue,\n  ssr_module: LiveVue.SSR.NodeJS,\n  ssr: true\n```\n\nUses elixir-nodejs with a pre-built server bundle for optimal performance.","ref":"configuration.html#ssr-modules"},{"type":"extras","title":"SSR Configuration - Configuration","doc":"Control SSR behavior globally or per-component:\n\n```elixir\n# Global SSR settings\nconfig :live_vue,\n  ssr: true,  # Enable SSR by default\n  ssr_filepath: \"./vue/server.js\"  # Server bundle path\n```","ref":"configuration.html#ssr-configuration"},{"type":"extras","title":"SSR Behavior - Configuration","doc":"SSR is intelligently applied:\n- **Runs during**: Initial page loads (dead renders)\n- **Skips during**: Live navigation and WebSocket updates\n- **Can be disabled**: Per-component with `v-ssr={false}`\n\nThis gives you the SEO and performance benefits of SSR without the overhead during live updates.","ref":"configuration.html#ssr-behavior"},{"type":"extras","title":"Per-Component SSR Control - Configuration","doc":"Override global settings for specific components:\n\n```elixir\n<!-- Force SSR for this component -->\n<.vue v-component=\"CriticalContent\" v-ssr={true} v-socket={@socket} />\n\n<!-- Disable SSR for client-only widgets -->\n<.vue v-component=\"InteractiveChart\" v-ssr={false} v-socket={@socket} />\n\n<!-- Use global default -->\n<.vue v-component=\"RegularComponent\" v-socket={@socket} />\n```","ref":"configuration.html#per-component-ssr-control"},{"type":"extras","title":"Testing Configuration - Configuration","doc":"LiveVue provides testing-specific configuration options to help with component testing and debugging.","ref":"configuration.html#testing-configuration"},{"type":"extras","title":"enable_props_diff - Configuration","doc":"By default, LiveVue optimizes performance by only sending prop changes (diffs) to the client. However, during testing, you may need access to the complete props state rather than just the incremental changes.\n\n```elixir\n# config/test.exs\nconfig :live_vue,\n  enable_props_diff: false\n```\n\nWhen disabled:\n- LiveVue will always send full props and not send diffs\n- The `props` field returned by `LiveVue.Test.get_vue/2` will contain the complete props state\n- This makes it easier to write comprehensive tests that verify the full component state\n- Useful for debugging component behavior and ensuring all props are correctly passed\n\n**Note**: This option is primarily intended for testing scenarios. In production, the default behavior (sending only diffs) provides better performance.","ref":"configuration.html#enable_props_diff"},{"type":"extras","title":"SSR Performance - Configuration","doc":"Vue SSR is compiled into optimized string concatenation for maximum performance. The SSR process:\n- Only runs during \"dead\" renders (no WebSocket connection)\n- Skips during live navigation for better UX\n- Can be disabled per-component when not needed","ref":"configuration.html#ssr-performance"},{"type":"extras","title":"Production SSR Setup - Configuration","doc":"For production deployments, you'll need Node.js 19+ and proper configuration:\n\n1. **Install Node.js 19+** in your production environment\n2. **Configure NodeJS supervisor** in your `application.ex`:\n\n```elixir\nchildren = [\n  {NodeJS.Supervisor, [path: LiveVue.SSR.NodeJS.server_path(), pool_size: 4]},\n  # ... other children\n]\n```\n\n3. **Build server bundle** as part of your deployment:\n\n```bash\n# In your deployment script\ncd assets && npm run build-server\n```\n\nThe server bundle will be created at `priv/vue/server.js` and used by the NodeJS supervisor.","ref":"configuration.html#production-ssr-setup"},{"type":"extras","title":"SSR Troubleshooting - Configuration","doc":"**SSR not working in development?**\n- Check that Vite dev server is running on the configured port\n- Verify `vite_host` matches your Vite server URL\n\n**SSR failing in production?**\n- Ensure Node.js 19+ is installed\n- Check that `priv/vue/server.js` exists after build\n- Verify NodeJS supervisor is properly configured\n\n**Performance issues?**\n- Consider adjusting the NodeJS pool size based on your server capacity\n- Disable SSR for components that don't benefit from it","ref":"configuration.html#ssr-troubleshooting"},{"type":"extras","title":"Troubleshooting Configuration - Configuration","doc":"","ref":"configuration.html#troubleshooting-configuration"},{"type":"extras","title":"Common Issues - Configuration","doc":"1. **Components not found**: Check `vue_root` paths in `LiveVue.Components`\n2. **SSR errors**: Verify `ssr_module` and `vite_host` configuration\n3. **TypeScript errors**: Ensure proper `tsconfig.json` setup\n4. **Build failures**: Check Vite configuration and entry points","ref":"configuration.html#common-issues"},{"type":"extras","title":"Debug Configuration - Configuration","doc":"Enable debug logging:\n\n```elixir\nconfig :logger, level: :debug\n\n# In your component\nrequire Logger\nLogger.debug(\"Vue component props: #{inspect(props)}\")\n\n# on the frontend, use VueDevTools to debug\n```","ref":"configuration.html#debug-configuration"},{"type":"extras","title":"Next Steps - Configuration","doc":"With your configuration complete, explore:\n- [Getting Started](getting_started.html) for your first component\n- [Basic Usage](basic_usage.html) for common patterns\n- [Advanced Features](advanced_features.html) for complex scenarios","ref":"configuration.html#next-steps"},{"type":"extras","title":"Component Reference","doc":"# Component Reference\n\nThis guide provides a complete reference for using Vue components in Phoenix LiveView templates.\n\n> #### Practical Examples {: .tip}\n>\n> For practical usage examples and patterns, see [Basic Usage](basic_usage.html). This reference focuses on complete syntax documentation.","ref":"component_reference.html"},{"type":"extras","title":"The .vue Component - Component Reference","doc":"The `.vue` component is the primary way to render Vue components in LiveView templates.","ref":"component_reference.html#the-vue-component"},{"type":"extras","title":"Basic Syntax - Component Reference","doc":"```elixir\n<.vue\n  v-component=\"ComponentName\"\n  v-socket={@socket}\n  prop_name={@value}\n/>\n```\n\nFor practical examples of different rendering patterns, see [Basic Usage](basic_usage.html#rendering-components).","ref":"component_reference.html#basic-syntax"},{"type":"extras","title":"Required Attributes - Component Reference","doc":"| Attribute | Type | Description | Example |\n|-----------|------|-------------|---------|\n| `v-component` | `string` | Vue component name or path | `\"Counter\"`, `\"admin/Dashboard\"` |\n| `v-socket` | `Phoenix.LiveView.Socket` | LiveView socket (required in LiveView) | `{@socket}` |","ref":"component_reference.html#required-attributes"},{"type":"extras","title":"Optional Attributes - Component Reference","doc":"| Attribute | Type | Default | Description |\n|-----------|------|---------|-------------|\n| `id` | `string` | auto-generated | Explicit wrapper element ID |\n| `class` | `string` | `nil` | CSS classes for wrapper element |\n| `v-ssr` | `boolean` | `true` | Enable/disable server-side rendering |\n| `v-diff` | `boolean` | `true` | Enable/disable props diffing for this component |","ref":"component_reference.html#optional-attributes"},{"type":"extras","title":"Event Handlers - Component Reference","doc":"Event handlers use the `v-on:` prefix to handle Vue component events.\n\n| Pattern | Description | Example |\n|---------|-------------|---------|\n| `v-on:event-name` | Handle Vue emit events | `v-on:save={JS.push(\"save\")}` |","ref":"component_reference.html#event-handlers"},{"type":"extras","title":"Props - Component Reference","doc":"All other attributes are passed as props to the Vue component. They are JSON-encoded - you need to either provide serializable types or implement [Jason.Encoder protocol](https://hexdocs.pm/jason/Jason.Encoder.html) for your custom types.\n\n```elixir\n<.vue\n  v-component=\"UserProfile\"\n  v-socket={@socket}\n  user={@user}\n  settings={@settings}\n  is_admin={@current_user.admin?}\n  count={42}\n  items={@list}\n/>\n```","ref":"component_reference.html#props"},{"type":"extras","title":"Component Shortcuts - Component Reference","doc":"When using `LiveVue.Components`, you can use shortcut syntax instead of the full `v-component` attribute.","ref":"component_reference.html#component-shortcuts"},{"type":"extras","title":"Setup - Component Reference","doc":"```elixir\n# In lib/my_app_web.ex\ndefp html_helpers do\n  quote do\n    use LiveVue.Components, vue_root: [\n      \"./assets/vue\",\n      \"./lib/my_app_web\"\n    ]\n  end\nend\n```","ref":"component_reference.html#setup"},{"type":"extras","title":"Usage - Component Reference","doc":"```elixir\n# Instead of\n<.vue v-component=\"Counter\" v-socket={@socket} count={@count} />\n\n# You can use\n<.Counter v-socket={@socket} count={@count} />\n```","ref":"component_reference.html#usage"},{"type":"extras","title":"Component Resolution - Component Reference","doc":"Components are resolved by file name.\n\n| File Path | Component Name | Shortcut |\n|-----------|----------------|----------|\n| `assets/vue/Counter.vue` | `\"Counter\"` | `<.Counter />` |\n| `assets/vue/admin/Dashboard.vue` | `\"Dashboard\"` | `<.Dashboard />` |\n\nFor components with identical names, use `.vue` component with unambiguous path:\n```elixir\n<.vue v-component=\"admin/Modal\" v-socket={@socket} />\n<.vue v-component=\"public/Modal\" v-socket={@socket} />\n```","ref":"component_reference.html#component-resolution"},{"type":"extras","title":"Slots - Component Reference","doc":"Vue components can receive slots from LiveView templates.","ref":"component_reference.html#slots"},{"type":"extras","title":"Basic Slots - Component Reference","doc":"```elixir\n<.vue v-component=\"Card\" v-socket={@socket}>\n   This content goes to the default slot \n  <.icon name=\"hero-star\" />\n</.vue>\n```\n\n```html\n<!-- Card.vue -->\n \n   \n      \n   \n \n```","ref":"component_reference.html#basic-slots"},{"type":"extras","title":"Named Slots - Component Reference","doc":"```elixir\n<.vue v-component=\"Modal\" v-socket={@socket}>\n  <:header>\n     Modal Title \n  </:header>\n\n   Modal content goes here \n\n  <:footer>\n     Cancel \n     Save \n  </:footer>\n</.vue>\n```\n\n```html\n<!-- Modal.vue -->\n \n   \n     \n        \n     \n\n     \n        \n     \n\n     \n        \n     \n   \n \n```","ref":"component_reference.html#named-slots"},{"type":"extras","title":"Slot Limitations - Component Reference","doc":"- Each slot is wrapped in a `div` element (technical limitation)\n- Slots are rendered server-side, so they can't contain Vue components\n- Phoenix hooks don't work inside slots\n- Slots remain reactive and update when their content changes","ref":"component_reference.html#slot-limitations"},{"type":"extras","title":"Event Handling - Component Reference","doc":"","ref":"component_reference.html#event-handling"},{"type":"extras","title":"Phoenix Events (Recommended) - Component Reference","doc":"Standard Phoenix events work directly in Vue components:\n\n```elixir\n<.vue v-component=\"Form\" v-socket={@socket}>\n  <!-- These work inside Vue components -->\n   Save \n   \n   ... \n</.vue>\n```\n\nFor more event handling patterns and examples, see [Basic Usage](basic_usage.html#handling-events).","ref":"component_reference.html#phoenix-events-recommended"},{"type":"extras","title":"Vue Event Handlers - Component Reference","doc":"Use `v-on:` for handling Vue component events:\n\n```elixir\n<.vue\n  v-component=\"Counter\"\n  v-socket={@socket}\n  count={@count}\n  v-on:increment={JS.push(\"inc\")}\n  v-on:decrement={JS.push(\"dec\")}\n  v-on:reset={JS.push(\"reset\")}\n/>\n```\n\nFor client-side event handling within Vue components, see [Client-Side API](client_api.html#uselivevue).","ref":"component_reference.html#vue-event-handlers"},{"type":"extras","title":"Event Payload Handling - Component Reference","doc":"When using `JS.push()` without a value, the emit payload is automatically used:\n\n```html\n<!-- Vue component -->\n Save \n```\n\n```elixir\n<!-- LiveView template -->\n<.vue v-on:save={JS.push(\"save\")} />\n<!-- Equivalent to: JS.push(\"save\", data: formData) -->\n```","ref":"component_reference.html#event-payload-handling"},{"type":"extras","title":"Complex Event Handlers - Component Reference","doc":"```elixir\n<.vue\n  v-component=\"DataTable\"\n  v-socket={@socket}\n  v-on:sort={JS.push(\"sort\") |> JS.patch(\"/sorted\")}\n  v-on:filter={JS.push(\"filter\") |> JS.show(to: \"#loading\")}\n  v-on:export={JS.push(\"export\") |> JS.navigate(\"/download\")}\n/>\n```","ref":"component_reference.html#complex-event-handlers"},{"type":"extras","title":"Server-Side Rendering (SSR) - Component Reference","doc":"","ref":"component_reference.html#server-side-rendering-ssr"},{"type":"extras","title":"Global SSR Configuration - Component Reference","doc":"For complete SSR configuration options, see [Configuration](configuration.html#server-side-rendering-ssr).","ref":"component_reference.html#global-ssr-configuration"},{"type":"extras","title":"Per-Component SSR Control - Component Reference","doc":"```elixir\n<!-- Enable SSR for this component -->\n<.vue v-component=\"HeavyComponent\" v-ssr={true} v-socket={@socket} />\n\n<!-- Disable SSR for this component -->\n<.vue v-component=\"ClientOnlyWidget\" v-ssr={false} v-socket={@socket} />\n\n<!-- Use global default -->\n<.vue v-component=\"RegularComponent\" v-socket={@socket} />\n```","ref":"component_reference.html#per-component-ssr-control"},{"type":"extras","title":"Props Diffing - Component Reference","doc":"LiveVue optimizes performance by sending only prop changes (diffs) instead of complete props on every update. This behavior can be controlled both globally and per-component.","ref":"component_reference.html#props-diffing"},{"type":"extras","title":"Global Configuration - Component Reference","doc":"For complete diffing configuration, see [Configuration - Testing Configuration](configuration.html#testing-configuration).\n\n```elixir\n# config/config.exs\nconfig :live_vue,\n  enable_props_diff: true  # Default: true\n```","ref":"component_reference.html#global-configuration"},{"type":"extras","title":"Per-Component Diff Control - Component Reference","doc":"Override global diffing settings for specific components:\n\n```elixir\n<!-- Enable diffing for this component (default) -->\n<.vue v-component=\"OptimizedComponent\" v-diff={true} v-socket={@socket} />\n\n<!-- Disable diffing - always send full props -->\n<.vue v-component=\"TestComponent\" v-diff={false} v-socket={@socket} />\n\n<!-- Use global default -->\n<.vue v-component=\"RegularComponent\" v-socket={@socket} />\n```","ref":"component_reference.html#per-component-diff-control"},{"type":"extras","title":"When to Disable Diffing - Component Reference","doc":"Consider disabling diffing for:\n- **Testing scenarios** where you need complete props state inspection\n- **Debugging complex prop updates** to see full component state\n- **Components with simple prop structures** where diffing overhead isn't beneficial\n- **Development troubleshooting** when diffing behavior is suspected of causing issues","ref":"component_reference.html#when-to-disable-diffing"},{"type":"extras","title":"Performance Impact - Component Reference","doc":"| Setting | Network Payload | Memory Usage | Update Speed |\n|---------|----------------|--------------|--------------|\n| `v-diff={true}` | Minimal (only changes) | Lower | Faster |\n| `v-diff={false}` | Larger (full props) | Higher | Slightly slower |","ref":"component_reference.html#performance-impact"},{"type":"extras","title":"SSR Behavior - Component Reference","doc":"| Context | SSR Enabled | Behavior |\n|---------|-------------|----------|\n| Initial page load | Yes | Component rendered on server |\n| Live navigation | No | Client-side rendering only |\n| WebSocket update | No | Client-side rendering only |\n| Dead view | Yes | Server-side rendering |","ref":"component_reference.html#ssr-behavior"},{"type":"extras","title":"Data Types and Serialization - Component Reference","doc":"","ref":"component_reference.html#data-types-and-serialization"},{"type":"extras","title":"Supported Prop Types - Component Reference","doc":"| Elixir Type | Vue Type | Example |\n|-------------|----------|---------|\n| `string` | `string` | `name={\"John\"}` |\n| `integer` | `number` | `count={42}` |\n| `float` | `number` | `price={19.99}` |\n| `boolean` | `boolean` | `active={true}` |\n| `list` | `array` | `items={[1, 2, 3]}` |\n| `map` | `object` | `user={%{name: \"John\"}}` |\n| `nil` | `null` | `optional={nil}` |","ref":"component_reference.html#supported-prop-types"},{"type":"extras","title":"Custom Structs with LiveVue.Encoder - Component Reference","doc":"When passing custom structs as props, you must implement the `LiveVue.Encoder` protocol to ensure they are properly serialized and can be diffed efficiently. This protocol is similar to `Jason.Encoder` but converts structs to maps instead of JSON strings.\n\n#### Deriving the Protocol\n\nThe simplest way to implement the protocol is by deriving it:\n\n```elixir\n# Encode all fields except :__struct__\ndefmodule User do\n  @derive LiveVue.Encoder\n  defstruct [:name, :email, :age]\nend\n\n# Encode only specific fields\ndefmodule User do\n  @derive {LiveVue.Encoder, only: [:name, :email]}\n  defstruct [:name, :email, :password, :admin_notes]\nend\n\n# Encode all fields except sensitive ones\ndefmodule User do\n  @derive {LiveVue.Encoder, except: [:password, :secret_key]}\n  defstruct [:name, :email, :password, :secret_key]\nend\n```\n\n#### Custom Implementation\n\nFor more control, implement the protocol manually:\n\n```elixir\ndefmodule User do\n  defstruct [:name, :email, :password, :profile]\nend\n\ndefimpl LiveVue.Encoder, for: User do\n  def encode(user, opts) do\n    %{\n      name: user.name,\n      email: user.email,\n      # Transform nested data as needed\n      profile: Map.take(user.profile, [:avatar, :bio])\n    }\n    |> LiveVue.Encoder.encode(opts)\n  end\nend\n```\n\n#### For Third-Party Structs\n\nIf you don't own the struct, use `Protocol.derive/3`:\n\n```elixir\n# In your application startup or module\nProtocol.derive(LiveVue.Encoder, SomeLibrary.User, only: [:id, :name])\n```\n\n#### Why LiveVue.Encoder is Required\n\nThe encoder protocol serves several important purposes:\n\n1. **Security**: Prevents accidental exposure of sensitive fields\n2. **Performance**: Enables efficient JSON patch diffing by converting structs to maps\n3. **Explicit Control**: Forces developers to be intentional about what data is sent to the client\n4. **Optimization**: Allows LiveVue to calculate minimal diffs for prop updates\n\nWithout implementing this protocol, you'll get a `Protocol.UndefinedError` when trying to pass custom structs as props.","ref":"component_reference.html#custom-structs-with-livevue-encoder"},{"type":"extras","title":"Complex Data Structures - Component Reference","doc":"```elixir\n<.vue\n  v-component=\"UserDashboard\"\n  v-socket={@socket}\n  user={%{\n    id: 1,\n    name: \"John Doe\",\n    email: \"john@example.com\",\n    preferences: %{\n      theme: \"dark\",\n      notifications: true\n    }\n  }}\n  permissions={[\"read\", \"write\", \"admin\"]}\n  metadata={%{\n    last_login: ~U[2023-01-01 12:00:00Z],\n    login_count: 42\n  }}\n/>\n```","ref":"component_reference.html#complex-data-structures"},{"type":"extras","title":"Date and Time Handling - Component Reference","doc":"```elixir\n# Dates are serialized as ISO strings\n<.vue\n  v-component=\"Calendar\"\n  v-socket={@socket}\n  current_date={Date.utc_today()}\n  created_at={DateTime.utc_now()}\n/>\n```\n\n```html\n<!-- Vue component -->\n \nconst props = defineProps<{\n  current_date: string  // \"2023-12-01\"\n  created_at: string    // \"2023-12-01T12:00:00Z\"\n}>()\n\n// Convert to JavaScript Date objects\nconst currentDate = new Date(props.current_date)\nconst createdAt = new Date(props.created_at)\n \n```","ref":"component_reference.html#date-and-time-handling"},{"type":"extras","title":"Error Handling - Component Reference","doc":"","ref":"component_reference.html#error-handling"},{"type":"extras","title":"Component Not Found - Component Reference","doc":"```elixir\n<!-- This will log an error to the console if Counter.vue doesn't exist -->\n<.vue v-component=\"Counter\" v-socket={@socket} />\n```","ref":"component_reference.html#component-not-found"},{"type":"extras","title":"Invalid Props - Component Reference","doc":"```elixir\n<!-- Vue will warn in console about type mismatches in development -->\n<.vue\n  v-component=\"UserProfile\"\n  v-socket={@socket}\n  user_id=\"not-a-number\"  <!-- Should be integer -->\n/>\n```","ref":"component_reference.html#invalid-props"},{"type":"extras","title":"SSR Errors - Component Reference","doc":"```elixir\n<!-- SSR errors are logged but don't break the page -->\n<.vue v-component=\"ProblematicComponent\" v-ssr={true} v-socket={@socket} />\n```","ref":"component_reference.html#ssr-errors"},{"type":"extras","title":"Performance Considerations - Component Reference","doc":"","ref":"component_reference.html#performance-considerations"},{"type":"extras","title":"Prop Optimization - Component Reference","doc":"```elixir\n# ✅ Good: Only pass what's needed\n<.vue\n  v-component=\"UserCard\"\n  v-socket={@socket}\n  user={%{name: @user.name, avatar: @user.avatar}}\n/>\n\n# ❌ Avoid: Passing large objects unnecessarily\n<.vue\n  v-component=\"UserCard\"\n  v-socket={@socket}\n  user={@user}  # Contains many unused fields\n/>\n```","ref":"component_reference.html#prop-optimization"},{"type":"extras","title":"Best Practices - Component Reference","doc":"","ref":"component_reference.html#best-practices"},{"type":"extras","title":"Component Organization - Component Reference","doc":"```elixir\n# ✅ Good: Descriptive component names\n<.vue v-component=\"UserProfileCard\" v-socket={@socket} />\n<.vue v-component=\"admin/UserManagementTable\" v-socket={@socket} />\n\n# ❌ Avoid: Generic names\n<.vue v-component=\"Component\" v-socket={@socket} />\n<.vue v-component=\"Widget\" v-socket={@socket} />\n```","ref":"component_reference.html#component-organization"},{"type":"extras","title":"Prop Naming - Component Reference","doc":"```elixir\n# ✅ Good: Clear, descriptive prop names\n<.vue\n  v-component=\"ProductCard\"\n  v-socket={@socket}\n  product_name={@product.name}\n  is_featured={@product.featured?}\n  price_in_cents={@product.price}\n/>\n\n# ❌ Avoid: Ambiguous prop names\n<.vue\n  v-component=\"ProductCard\"\n  v-socket={@socket}\n  name={@product.name}\n  flag={@product.featured?}\n  amount={@product.price}\n/>\n```","ref":"component_reference.html#prop-naming"},{"type":"extras","title":"Event Naming - Component Reference","doc":"```elixir\n# ✅ Good: Descriptive event names\n<.vue\n  v-component=\"ShoppingCart\"\n  v-socket={@socket}\n  v-on:item-added={JS.push(\"add_item\")}\n  v-on:checkout-started={JS.push(\"start_checkout\")}\n/>\n\n# ❌ Avoid: Generic event names\n<.vue\n  v-component=\"ShoppingCart\"\n  v-socket={@socket}\n  v-on:click={JS.push(\"handle_click\")}\n  v-on:action={JS.push(\"do_action\")}\n/>\n```","ref":"component_reference.html#event-naming"},{"type":"extras","title":"Common Patterns - Component Reference","doc":"","ref":"component_reference.html#common-patterns"},{"type":"extras","title":"Data Display Components - Component Reference","doc":"```elixir\n<.vue\n  v-component=\"DataTable\"\n  v-socket={@socket}\n  items={@users}\n  columns={[\"name\", \"email\", \"created_at\"]}\n  sortable={true}\n  v-on:sort={JS.push(\"sort_users\")}\n  v-on:filter={JS.push(\"filter_users\")}\n/>\n```","ref":"component_reference.html#data-display-components"},{"type":"extras","title":"Troubleshooting - Component Reference","doc":"","ref":"component_reference.html#troubleshooting"},{"type":"extras","title":"Component Not Rendering - Component Reference","doc":"1. Check component name spelling and case sensitivity\n2. Verify `v-socket={@socket}` is present in LiveView\n3. Ensure component file exists in configured paths\n4. Check browser console for JavaScript errors","ref":"component_reference.html#component-not-rendering"},{"type":"extras","title":"Props Not Updating - Component Reference","doc":"1. Verify prop names match between LiveView and Vue\n2. Check that LiveView assigns are actually changing\n3. Ensure `@socket` is connected (not in dead view)","ref":"component_reference.html#props-not-updating"},{"type":"extras","title":"Events Not Working - Component Reference","doc":"1. Check event name spelling in both Vue emit and LiveView handler\n2. Verify `handle_event/3` function exists in LiveView\n3. Check event payload structure matches expectations","ref":"component_reference.html#events-not-working"},{"type":"extras","title":"Next Steps - Component Reference","doc":"- [Basic Usage](basic_usage.html) for practical patterns and examples\n- [Client-Side API](client_api.html) for Vue component development\n- [Configuration](configuration.html) for advanced setup and customization\n- [Testing](testing.html) for testing component integration","ref":"component_reference.html#next-steps"},{"type":"extras","title":"Client-Side API","doc":"# Client-Side API Reference\n\nThis guide documents all client-side utilities, composables, and APIs available in LiveVue for Vue components.\n\n> #### Getting Started {: .tip}\n>\n> New to LiveVue? Check out [Basic Usage](basic_usage.html) for fundamental patterns before diving into the API details.","ref":"client_api.html"},{"type":"extras","title":"Composables - Client-Side API","doc":"LiveVue provides several Vue Composables to make interacting with the LiveView instance from your components easier and more declarative. These should be your first choice when working inside `.vue` files.\n\n### `useLiveVue()`\n\nThe `useLiveVue()` composable is the standard way to get access to the LiveView hook instance within your component's ` ` block. This is the method you should use when you need to call hook methods from your script logic (e.g., in watchers, lifecycle hooks, or other functions).\n\n```html\n \nimport { useLiveVue } from 'live_vue'\n\n// Must be called inside setup\nconst live = useLiveVue()\n\n// Now you can use `live` anywhere in your script\nlive.pushEvent(\"some_event\")\n \n```\n\n### `useLiveEvent(event, callback)`\n\nThe `useLiveEvent` composable is the recommended way to listen for server-pushed events within a component. It automatically registers an event handler when the component is mounted and cleans it up when the component is unmounted.\n\nIt is a wrapper around `useLiveVue().handleEvent()` that saves you the boilerplate of using `onMounted` and `onUnmounted`.\n\n**Parameters:**\n- `event` (string): Event name to listen for\n- `callback` (function): Handler function that receives the payload from the server.\n\n**Example: Showing Notifications**\n\n```html\n \nimport { useLiveEvent } from 'live_vue'\nimport { useToast } from 'primevue/usetoast'; // example toast library\n\nconst toast = useToast();\n\nuseLiveEvent(\"notification\", (payload) => {\n  toast.add({ severity: payload.type, summary: payload.message, life: 3000 });\n})\n \n```\n\n### `useLiveNavigation()`\n\nA composable for programmatic navigation that mirrors the functionality of `live_patch` and `live_redirect` in Phoenix LiveView. It returns an object with `patch` and `navigate` functions.\n\nThis is useful for scenarios where you need to trigger navigation from your script logic, such as after a form submission or a modal action.\n\n**Returns:**\n- `patch(hrefOrQueryParams, { replace: boolean })`: Patches the current LiveView, similar to `live_patch`.\n- `navigate(href, { replace: boolean })`: Navigates to a new LiveView, similar to `live_redirect`.\n\n**Example: Navigating after a successful action**\n\n```html\n \n   \n     Go to Settings \n   \n \n\n \nimport { useLiveNavigation } from 'live_vue';\nimport { useLiveEvent } from 'live_vue';\n\nconst { patch, navigate } = useLiveNavigation();\n\nuseLiveEvent(\"user_created\", (payload) => {\n  // Redirect to the new user's page\n  navigate(`/users/${payload.id}`);\n})\n\nfunction goToSettings() {\n  // Update the URL with new query params\n  patch({ tab: 'settings', page: 1 });\n}\n \n```","ref":"client_api.html#composables"},{"type":"extras","title":"Low-Level API - Client-Side API","doc":"While composables are recommended for most component-based use cases, you can also access the underlying hook instance for more control or for use outside of components.","ref":"client_api.html#low-level-api"},{"type":"extras","title":"Accessing the Hook Instance - Client-Side API","doc":"There are two primary ways to interact with the LiveView instance from your Vue component:\n\n#### 1. `useLiveVue()` Composable\n\nAs seen above, `useLiveVue()` returns the raw hook instance.\n\n#### 2. `$live` Global Property\n\nFor convenience, the LiveView hook instance is also exposed directly to your Vue templates as a global property named `$live`. This is ideal for simple, one-off calls directly from an element's event handler, as it saves you from importing and calling `useLiveVue()` when you don't need the instance in your script.\n\n```html\n \n  <!-- No script setup needed for this simple case -->\n   \n    Click Me\n   \n \n```\n\nBoth `useLiveVue()` and `$live` return the same hook instance, which is fully typed and provides access to the methods below.","ref":"client_api.html#accessing-the-hook-instance"},{"type":"extras","title":"Hook Methods - Client-Side API","doc":"The hook instance (returned by `useLiveVue()` or accessed via `$live`) provides the following methods:\n\n##### pushEvent(event, payload?, callback?)\n\nPush an event to the LiveView server.\n\n```html\n \nimport { useLiveVue } from 'live_vue'\nconst live = useLiveVue()\n\n// Basic usage - increment a counter\nlive.pushEvent(\"increment\", { amount: 1 })\n\n// Form submission with validation feedback\nlive.pushEvent(\"save_user\", {\n  name: \"John\",\n  email: \"john@example.com\"\n}, (reply, ref) => {\n  if (reply.status === \"ok\") {\n    console.log(\"User saved successfully!\")\n  } else {\n    console.log(\"Validation errors:\", reply.errors)\n  }\n})\n\n// Simple refresh without payload\nlive.pushEvent(\"refresh\")\n \n```\n\n**Real-world example - Auto-save draft:**\n```html\n \nimport { watch, debounce } from 'vue'\nimport { useLiveVue } from 'live_vue'\n\nconst live = useLiveVue()\nconst content = ref('')\n\n// Auto-save draft every 2 seconds after user stops typing\nconst debouncedSave = debounce((text) => {\n  live.pushEvent(\"save_draft\", { content: text })\n}, 2000)\n\nwatch(content, debouncedSave)\n \n```\n\n**Parameters:**\n- `event` (string): Event name to push to LiveView\n- `payload` (object, optional): Data to send with the event\n- `callback` (function, optional): Callback for server replies\n\n**Returns:** Event reference number\n\n##### handleEvent(event, callback)\n\nListen for events pushed from the LiveView server.\n\n> **Note:** When using this inside a Vue component, prefer the [`useLiveEvent`](#useliveevent) composable for automatic cleanup. Use `handleEvent` when you need to manually manage the listener's lifecycle.\n\n```html\n \nimport { onMounted, onUnmounted } from 'vue'\nimport { useLiveVue } from 'live_vue'\n\nconst live = useLiveVue()\n\n// Listen for server-sent notifications\nconst callbackRef = live.handleEvent(\"notification\", (payload) => {\n  showToast(payload.message, payload.type)\n})\n\n// Clean up on unmount to prevent memory leaks\nonUnmounted(() => {\n  live.removeHandleEvent(callbackRef)\n})\n \n```\n\n**Parameters:**\n- `event` (string): Event name to listen for\n- `callback` (function): Handler function receiving the payload\n\n**Returns:** A reference to the callback that can be used with `removeHandleEvent`.\n\n##### removeHandleEvent(callbackRef)\n\nRemoves an event listener that was previously registered with `handleEvent`.\n\n```js\n// callbackRef is the value returned from a `handleEvent` call\nlive.removeHandleEvent(callbackRef)\n```\n\n**Parameters:**\n- `callbackRef`: The reference returned by `handleEvent`.\n\n##### pushEventTo(selector, event, payload?, callback?)\n\nPush an event to a specific LiveView component.\n\n```html\n \nimport { useLiveVue } from 'live_vue'\nconst live = useLiveVue()\n\n// Push to specific form component\nlive.pushEventTo(\"#user-form\", \"validate\", formData)\n\n// Target component by data attribute\nlive.pushEventTo(\"[data-component='UserProfile']\", \"refresh\")\n\n \n```\n\n##### upload(name, entries)\n\nHandle file uploads to LiveView.\n\n```html\n \nimport { useLiveVue } from 'live_vue'\nconst live = useLiveVue()\n\n// Handle file upload\nconst fileInput = ref ()\n\nconst handleUpload = () => {\n  if (fileInput.value?.files) {\n    live.upload(\"avatar\", fileInput.value.files)\n  }\n}\n \n```\n\n**Real-world example - Drag & drop upload:**\n```html\n \nimport { ref } from 'vue'\nimport { useLiveVue } from 'live_vue'\n\nconst live = useLiveVue()\nconst isDragging = ref(false)\n\nconst handleDrop = (event) => {\n  event.preventDefault()\n  isDragging.value = false\n\n  const files = event.dataTransfer.files\n  if (files.length > 0) {\n    live.upload(\"documents\", files)\n  }\n}\n\nconst handleDragOver = (event) => {\n  event.preventDefault()\n  isDragging.value = true\n}\n \n\n \n   \n    Drop files here to upload\n   \n \n```\n\n**Parameters:**\n- `name` (string): Upload name (must match LiveView allow_upload)\n- `entries` (FileList): Files to upload\n\n##### uploadTo(selector, name, entries)\n\nUpload files to a specific LiveView component.\n\n```html\n \nimport { useLiveVue } from 'live_vue'\nconst live = useLiveVue()\n\nlive.uploadTo(\"#profile-form\", \"avatar\", files)\n \n```","ref":"client_api.html#hook-methods"},{"type":"extras","title":"Built-in Components - Client-Side API","doc":"","ref":"client_api.html#built-in-components"},{"type":"extras","title":"Link - Client-Side API","doc":"The `Link` component provides a convenient wrapper around Phoenix LiveView's navigation capabilities, making it easy to perform `patch` and `navigate` actions from within your Vue components.\n\n```html\n \nimport { Link } from 'live_vue'\n \n\n \n  <!-- Basic link -->\n   Regular Link \n\n  <!-- `live_patch` to the same LiveView -->\n   Edit Post \n\n  <!-- `live_redirect` to a different LiveView -->\n   Back to Posts \n\n  <!-- Replace the current history entry -->\n   Edit (replace history) \n \n```\n\n#### Props\n\n| Prop | Type | Description |\n|---|---|---|\n| `href` | `string` | A standard link that causes a full page reload. |\n| `patch` | `string` | Navigates to a new URL within the same LiveView by calling `handle_params`. |\n| `navigate` | `string` | Navigates to a different LiveView, replacing the current one without a full page reload. |\n| `replace` | `boolean` | If `true`, the browser's history entry is replaced instead of a new one being pushed. |","ref":"client_api.html#link"},{"type":"extras","title":"Utility Functions - Client-Side API","doc":"","ref":"client_api.html#utility-functions"},{"type":"extras","title":"createLiveVue(config) - Client-Side API","doc":"Creates a LiveVue application instance. For complete configuration options, see [Configuration](configuration.html#vue-application-setup).","ref":"client_api.html#createlivevue-config"},{"type":"extras","title":"findComponent(components, name) - Client-Side API","doc":"A flexible helper function to resolve a component from a map of available components. It finds a component by checking if the key ends with either `name.vue` or `name/index.vue`.\n\nThis is particularly useful when using Vite's `import.meta.glob` to import all components, as it allows for a simple and conventional way to organize and resolve them.\n\n```typescript\nimport { findComponent } from 'live_vue'\n\n// Given a components map from Vite:\nconst components = import.meta.glob([\"./**/*.vue\", \"../../lib/**/*.vue\"]);\n\n// It can resolve the following:\n// 1. By component name: `findComponent(components, 'UserProfile')`\n//    -> Matches: `./components/UserProfile.vue`\n// 2. By path: `findComponent(components, 'admin/Dashboard')`\n//    -> Matches: `./components/admin/Dashboard.vue`\n// 3. By directory with an index file: `findComponent(components, 'forms/Button')`\n//    -> Matches: `./components/forms/Button/index.vue`\n```\n\nIf the component is not found, it will throw a helpful error listing all available components.\n\n**Parameters:**\n- `components` (object): A map of component paths to component modules, typically from `import.meta.glob`.\n- `name` (string): The name or path of the component to find.\n\n**Returns:** The resolved Vue component, or throws an error if not found.","ref":"client_api.html#findcomponent-components-name"},{"type":"extras","title":"getHooks(liveVueApp) - Client-Side API","doc":"Generates Phoenix LiveView hooks for LiveVue integration.\n\n```typescript\nimport { getHooks } from 'live_vue'\nimport liveVueApp from '../vue'\n\nconst hooks = getHooks(liveVueApp)\n\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  hooks: hooks\n})\n```","ref":"client_api.html#gethooks-livevueapp"},{"type":"extras","title":"TypeScript Support - Client-Side API","doc":"","ref":"client_api.html#typescript-support"},{"type":"extras","title":"Type Definitions - Client-Side API","doc":"LiveVue provides comprehensive TypeScript support:\n\n```typescript\n// Component props typing\ninterface Props {\n  user: {\n    id: number\n    name: string\n    email: string\n  }\n  settings: Record \n}\n\nconst props = defineProps ()\n\n// Event emissions typing\nconst emit = defineEmits<{\n  'user-updated': [{ user: Props['user'] }]\n  'settings-changed': [{ key: string, value: any }]\n}>()\n\n// useLiveVue typing\nconst live = useLiveVue()\n// live is fully typed with all available methods\n```","ref":"client_api.html#type-definitions"},{"type":"extras","title":"Common Patterns - Client-Side API","doc":"","ref":"client_api.html#common-patterns"},{"type":"extras","title":"Real-Time Data Synchronization - Client-Side API","doc":"```typescript\n// Composable for real-time data sync with {:reply, data, socket} tuple\nexport const useRealtimeData = (dataType: string) => {\n  const live = useLiveVue()\n  const data = ref(null)\n  const loading = ref(true)\n  const error = ref(null)\n\n  onMounted(() => {\n    // Request initial data\n    live.pushEvent(\"load_data\", { type: dataType }, (newData) => {\n      data.value = newData\n      loading.value = false\n    })\n  })\n\n  return { data, loading, error }\n}\n```","ref":"client_api.html#real-time-data-synchronization"},{"type":"extras","title":"File Upload Helper - Client-Side API","doc":"```typescript\n// Composable for file uploads\nexport const useFileUpload = (uploadName: string) => {\n  const live = useLiveVue()\n  const uploading = ref(false)\n  const progress = ref(0)\n\n  const upload = (files: FileList) => {\n    uploading.value = true\n    progress.value = 0\n\n    // Listen for upload progress\n    live.handleEvent(\"upload_progress\", ({ percentage }) => {\n      progress.value = percentage\n    })\n\n    // Listen for upload completion\n    live.handleEvent(\"upload_complete\", () => {\n      uploading.value = false\n      progress.value = 100\n    })\n\n    live.upload(uploadName, files)\n  }\n\n  return { upload, uploading, progress }\n}\n```","ref":"client_api.html#file-upload-helper"},{"type":"extras","title":"Performance Considerations - Client-Side API","doc":"","ref":"client_api.html#performance-considerations"},{"type":"extras","title":"Debounced Events - Client-Side API","doc":"```html\n \nimport { useLiveVue } from 'live_vue'\nimport { debounce } from 'lodash-es'\n\nconst live = useLiveVue()\n\n// Debounce search input\nconst debouncedSearch = debounce((query: string) => {\n  live.pushEvent(\"search\", { query })\n}, 300)\n\nwatch(searchQuery, debouncedSearch)\n \n```","ref":"client_api.html#debounced-events"},{"type":"extras","title":"Event Cleanup - Client-Side API","doc":"```html\n \nimport { useLiveVue } from 'live_vue'\nimport { onMounted, onUnmounted } from 'vue'\n\nconst live = useLiveVue()\n\n// Helper function to clean up event listeners\n// will likely be added to LiveVue in the future\nexport function useLiveEvent(event, callback) {\n  let callbackRef = null\n  onMounted(() => {\n    callbackRef = live.handleEvent(event, callback)\n  })\n  onUnmounted(() => {\n    if (callbackRef) live.removeHandleEvent(callbackRef)\n    callbackRef = null\n  })\n}\n\n\nuseLiveEvent(\"data_update\", (data) => console.log(\"Data updated:\", data))\n \n```","ref":"client_api.html#event-cleanup"},{"type":"extras","title":"Next Steps - Client-Side API","doc":"- [Basic Usage](basic_usage.html) for fundamental patterns and examples\n- [Component Reference](component_reference.html) for LiveView-side API\n- [Configuration](configuration.html) for advanced setup options\n- [Testing](testing.html) for testing client-side code","ref":"client_api.html#next-steps"},{"type":"extras","title":"How LiveVue Works","doc":"# How LiveVue Works\n\nThis guide explains the architecture and inner workings of LiveVue, helping you understand the design decisions and implementation details.\n\n> #### Practical Usage {: .tip}\n>\n> Looking for practical examples? Check out [Basic Usage](basic_usage.html) for common patterns and [Getting Started](getting_started.html) for your first component.","ref":"architecture.html"},{"type":"extras","title":"Overview - How LiveVue Works","doc":"LiveVue bridges two different paradigms: **Phoenix LiveView** with its server-side state management and HTML over WebSockets, and **Vue.js** with its client-side reactivity and virtual DOM. The challenge is making these two systems work together seamlessly while maintaining the benefits of both.","ref":"architecture.html#overview"},{"type":"extras","title":"Architecture Diagram - How LiveVue Works","doc":"![LiveVue flow](./images/lifecycle.png)","ref":"architecture.html#architecture-diagram"},{"type":"extras","title":"Component Lifecycle - How LiveVue Works","doc":"","ref":"architecture.html#component-lifecycle"},{"type":"extras","title":"1. Server-Side Rendering (SSR) - How LiveVue Works","doc":"When a LiveView renders a Vue component, LiveVue generates a special `div` element with component configuration stored in data attributes. Here's what the server generates:\n\n```elixir\n# In your LiveView template\n<.vue v-component=\"MyComponent\" message={@message} v-on:click=\"handle_click\" />\n```\n\nThis produces HTML like:\n\n```html\n \n  <!-- Optional SSR content here -->\n \n```\n\nThe component name, props serialized as JSON, event handlers, and slots are all embedded as data attributes, with an optional Phoenix LiveView hook attachment.","ref":"architecture.html#1-server-side-rendering-ssr"},{"type":"extras","title":"2. Client-Side Hydration - How LiveVue Works","doc":"When the page loads and Phoenix LiveView connects, the `VueHook` activates. Here's the simplified flow from `hooks.ts`:\n\n```typescript\nexport const getVueHook = ({ resolve, setup }: LiveVueApp): LiveHookInternal => ({\n  async mounted() {\n    const componentName = this.el.getAttribute(\"data-name\") as string\n    const component = await resolve(componentName)\n\n    const props = reactive(getProps(this.el, this.liveSocket))\n    const slots = reactive(getSlots(this.el))\n\n    const app = setup({\n      createApp: makeApp,\n      component,\n      props,\n      slots,\n      // ... other options\n    })\n\n    this.vue = { props, slots, app }\n  }\n})\n```\n\nThe hook resolves the component name to the actual Vue component, makes props and slots reactive using Vue's reactivity system, mounts the Vue component (optionally hydrating existing SSR content), and configures event handlers for bidirectional communication.","ref":"architecture.html#2-client-side-hydration"},{"type":"extras","title":"3. Reactive Updates - How LiveVue Works","doc":"When server state changes, LiveView sends new data via WebSocket. Phoenix updates only the changed data attributes, Vue's reactivity system automatically detects these changes, and only affected parts of the Vue component re-render. This happens through the `updated()` hook:\n\n```typescript\nupdated() {\n  Object.assign(this.vue.props ?? {}, getProps(this.el, this.liveSocket))\n  Object.assign(this.vue.slots ?? {}, getSlots(this.el))\n}\n```\n\n#### Props Diff Construction (Server-Side)\n\nLiveVue implements an efficient diffing system that minimizes data transmission by sending only changed properties as JSON patches. Here's how it works:\n\n**1. Change Detection**\nLiveView's `__changed__` tracking system identifies which assigns have been modified since the last render. For simple value changes, `__changed__` contains `true` for the changed key. For complex data structures (maps, lists, structs), it stores the previous value to enable deep diffing.\n\n**2. Struct Encoding**\nBefore diffing can occur, any custom structs are converted to maps using the `LiveVue.Encoder` protocol. This ensures consistent data structures that can be reliably compared and serialized.\n\n**3. Diff Calculation**\nThe system processes each changed prop differently based on its complexity:\n\n- **Simple values** (strings, numbers, booleans): Generate a direct \"replace\" operation since the entire value has changed\n- **Complex values** (maps, lists, structs): Use the Jsonpatch library to calculate minimal differences between the old and new encoded values\n\n**4. Path Construction**\nEach diff operation includes a JSON Pointer path that precisely identifies where the change should be applied. For nested changes, paths are constructed hierarchically (e.g., `/user/email` for changing a user's email field).\n\n**5. Serialization**\nThe resulting diff operations are JSON-encoded and embedded in the `data-props-diff` attribute of the component's wrapper div. This attribute is only present when there are actual changes to apply.\n\n#### Props Diff Consumption (Client-Side)\n\nOn the client side, the Vue hook processes these diffs efficiently:\n\n**1. Diff Extraction**\nWhen the `updated()` hook fires, it reads the `data-props-diff` attribute and parses the JSON array of patch operations.\n\n**2. Patch Application**\nThe system applies each patch operation to the reactive props object using a JSON Patch implementation. This directly modifies the existing props object rather than replacing it entirely.\n\n**3. Reactivity Triggering**\nSince the props object is reactive (created with Vue's `reactive()` function), any changes automatically trigger Vue's reactivity system. Components re-render only the parts of the template that depend on the changed data.\n\n**4. Efficient Updates**\nThis approach means that even for large, complex data structures, only the specific fields that changed are updated in the DOM. For example, changing a user's email in a large user object only updates the email display, not the entire user profile.\n\n**Performance Benefits**\nThis diff-based approach provides several advantages:\n- Minimal network payload (only changed data is transmitted)\n- Efficient client-side updates (only changed reactive properties trigger re-renders)\n- Reduced memory pressure (existing objects are patched rather than replaced)\n- Faster UI updates (smaller changes mean less work for Vue's virtual DOM)\n\nThe combination of server-side diff calculation and client-side patch application ensures that LiveVue can handle complex, nested data structures efficiently while maintaining real-time reactivity.\n\n**Disabling Diffs**\nFor testing scenarios or debugging purposes, diffing can be disabled globally via the `enable_props_diff: false` configuration option, or per-component using the `v-diff={false}` attribute. When disabled, complete props are always sent instead of diffs, which can be useful for comprehensive testing or troubleshooting complex prop updates. See [Configuration](configuration.html#testing-configuration) for details.","ref":"architecture.html#3-reactive-updates"},{"type":"extras","title":"Data Flow - How LiveVue Works","doc":"","ref":"architecture.html#data-flow"},{"type":"extras","title":"Props Flow (Server → Client) - How LiveVue Works","doc":"LiveView manages authoritative state and passes it to Vue components as props. When LiveView assigns are updated, the HEEX template generates new prop data, only changed props are sent over WebSocket, and the Vue component automatically re-renders with new props.\n\nThe server-side extraction logic in `live_vue.ex` ensures efficient updates:\n\n```elixir\ndefp extract(assigns, type) do\n  Enum.reduce(assigns, %{}, fn {key, value}, acc ->\n    case normalize_key(key, value) do\n      ^type -> Map.put(acc, key, value)\n      {^type, k} -> Map.put(acc, k, value)\n      _ -> acc\n    end\n  end)\nend\n```\n\n#### Struct Encoding with LiveVue.Encoder\n\nBefore props are serialized and sent to the client, custom structs must be encoded using the `LiveVue.Encoder` protocol. This protocol:\n\n1. **Converts structs to maps** for JSON serialization\n2. **Enables efficient diffing** by providing a consistent data structure\n3. **Ensures security** by requiring explicit field exposure\n4. **Optimizes performance** by allowing minimal JSON patches\n\n```elixir\n# Example: User struct with encoder protocol\ndefmodule User do\n  @derive {LiveVue.Encoder, except: [:password]}\n  defstruct [:name, :email, :password, :created_at]\nend\n\n# When passed as props:\n<.vue user={@current_user} v-component=\"Profile\" v-socket={@socket} />\n\n# The encoder converts the struct to:\n%{\n  name: \"John Doe\",\n  email: \"john@example.com\",\n  created_at: ~U[2023-01-01 12:00:00Z]\n  # password field is excluded for security\n}\n```\n\nThe encoding happens in the `vue/1` component function:\n\n```elixir\nchanged_props = extract(changed, :props) |> Encoder.encode()\n```\n\nThis ensures that all props are in a format suitable for JSON serialization and efficient diffing.","ref":"architecture.html#props-flow-server-client"},{"type":"extras","title":"Event Flow (Client → Server) - How LiveVue Works","doc":"There are three main approaches for handling events:\n\n**Standard Phoenix Events** (recommended for most cases) use direct `phx-click` attributes that work inside Vue components:\n\n```html\n Click me \n```\n\n**Programmatic Events** use `useLiveVue().pushEvent()` for complex logic:\n\n```javascript\nconst live = useLiveVue()\nlive.pushEvent(\"custom_event\", { data: \"value\" })\n```\n\n**Vue Event Handlers** use the `v-on:` syntax for reusable components:\n\n```elixir\n<.vue v-component=\"Counter\" v-on:increment=\"handle_increment\" />\n```\n\nThe event handlers are processed on the client side by invoking `liveSocket.execJS` with the payload defined by `JS` module.\n\n> #### Event Handling Best Practices {: .tip}\n>\n> Use `phx-click` for simple, direct event handling. Use `live.pushEvent()` when you need programmatic control or complex logic. Use `v-on:` syntax when creating reusable Vue components that should be decoupled from specific LiveView implementations.","ref":"architecture.html#event-flow-client-server"},{"type":"extras","title":"Key Design Decisions - How LiveVue Works","doc":"","ref":"architecture.html#key-design-decisions"},{"type":"extras","title":"Hook-Based Integration - How LiveVue Works","doc":"LiveVue uses Phoenix LiveView's hook system rather than a separate JavaScript framework. This provides seamless integration within LiveView's lifecycle, automatic cleanup when elements are removed, and natural compatibility with all Phoenix events.","ref":"architecture.html#hook-based-integration"},{"type":"extras","title":"Reactive Props and Slots - How LiveVue Works","doc":"Props and slots are made reactive using Vue's reactivity system, enabling efficient updates where only changed data triggers re-renders, full compatibility with Vue features like computed properties and watchers, and minimal overhead for prop updates.","ref":"architecture.html#reactive-props-and-slots"},{"type":"extras","title":"Performance Optimizations - How LiveVue Works","doc":"","ref":"architecture.html#performance-optimizations"},{"type":"extras","title":"Selective Updates - How LiveVue Works","doc":"LiveVue minimizes data transmission by tracking only modified props, slots, and handlers. The JSON encoding is optimized to prevent redundant work, and Phoenix updates only specific data attributes rather than re-rendering entire elements.\n\n#### Efficient Struct Diffing\n\nThe `LiveVue.Encoder` protocol enables efficient diffing of complex data structures:\n\n```elixir\n# For complex types, use Jsonpatch to find minimal diff\nold_value ->\n  old_value\n  |> Encoder.encode()\n  |> Jsonpatch.diff(new_value)\n  |> update_in([Access.all(), :path], fn path -> \"/#{k}#{path}\" end)\n```\n\nBy converting structs to consistent map representations, LiveVue can:\n- Calculate minimal JSON patches for prop updates\n- Avoid sending unchanged nested data\n- Reduce WebSocket payload sizes\n- Improve client-side rendering performance\n\nFor example, when only a user's email changes:\n\n```elixir\n# Instead of sending the entire user struct\n%{user: %{name: \"John\", email: \"new@example.com\", created_at: ~U[...]}}\n\n# LiveVue sends only the changed field\n[%{op: \"replace\", path: \"/user/email\", value: \"new@example.com\"}]\n```","ref":"architecture.html#selective-updates"},{"type":"extras","title":"SSR Optimization - How LiveVue Works","doc":"Server-side rendering is intelligently applied only during initial page loads (dead renders), can be configured per component, and is skipped during live navigation for better performance.","ref":"architecture.html#ssr-optimization"},{"type":"extras","title":"Automatic Preloading - How LiveVue Works","doc":"During server-side rendering, LiveVue automatically uses the Vite-generated manifest file to inject resource preload links (` ` and others) for all the assets required by a component. This ensures that the browser can download necessary JavaScript and CSS files earlier in the page load process, improving perceived performance and reducing the time to an interactive page.","ref":"architecture.html#automatic-preloading"},{"type":"extras","title":"Memory Management - How LiveVue Works","doc":"Automatic cleanup prevents memory leaks through proper hook lifecycle management. Vue apps are unmounted when hooks are destroyed, with special handling for Phoenix navigation events and automatic removal of event listeners:\n\n```typescript\ndestroyed() {\n  const instance = this.vue.app\n  if (instance) {\n    window.addEventListener(\"phx:page-loading-stop\", () => instance.unmount(), { once: true })\n  }\n}\n```","ref":"architecture.html#memory-management"},{"type":"extras","title":"Security Considerations - How LiveVue Works","doc":"","ref":"architecture.html#security-considerations"},{"type":"extras","title":"Data Sanitization - How LiveVue Works","doc":"All data passed between server and client is properly sanitized. Props are safely encoded with HTML escaping using `Jason.encode!(data, escape: :html_safe)`, all user data is escaped before transmission, and events go through Phoenix's standard validation.","ref":"architecture.html#data-sanitization"},{"type":"extras","title":"Event Security - How LiveVue Works","doc":"Event handling maintains Phoenix's security model where all events are validated on the server, standard Phoenix CSRF protection applies, and LiveView's authorization patterns work normally.","ref":"architecture.html#event-security"},{"type":"extras","title":"Slots Implementation - How LiveVue Works","doc":"Slots bridge HEEX templates and Vue components by rendering slots server-side as HTML, encoding content as Base64 for safe transport, and decoding on the client for integration into Vue's slot system:\n\n```typescript\nconst getSlots = (el: HTMLElement): Record  => {\n  const dataSlots = getAttributeJson(el, \"data-slots\")\n  return mapValues(dataSlots, base64 => () => h(\"div\", { innerHTML: atob(base64).trim() }))\n}\n```\n\n**Limitation**: Since slots are rendered server-side, they can't contain other Vue components or Phoenix hooks.","ref":"architecture.html#slots-implementation"},{"type":"extras","title":"Debugging and Development - How LiveVue Works","doc":"","ref":"architecture.html#debugging-and-development"},{"type":"extras","title":"Development Tools - How LiveVue Works","doc":"LiveVue works with standard development tools including Vue DevTools for full component inspection and debugging, Phoenix LiveView Dashboard for server-side state monitoring, and browser DevTools for network and WebSocket inspection.","ref":"architecture.html#development-tools"},{"type":"extras","title":"Debug Features - How LiveVue Works","doc":"Built-in debugging capabilities include debug mode for detailed logging of component lifecycle, component resolution logs to help identify loading issues, and event tracing to track events flowing between Vue and LiveView.","ref":"architecture.html#debug-features"},{"type":"extras","title":"Limitations and Trade-offs - How LiveVue Works","doc":"","ref":"architecture.html#limitations-and-trade-offs"},{"type":"extras","title":"Current Limitations - How LiveVue Works","doc":"Vue components can't contain other Vue components, Phoenix hooks don't work inside slots, and there's limited browser API access during server rendering.","ref":"architecture.html#current-limitations"},{"type":"extras","title":"Design Trade-offs - How LiveVue Works","doc":"The Vue runtime adds approximately 34KB gzipped to your application. There's an additional abstraction layer between Phoenix and the client, and it requires understanding both Phoenix LiveView and Vue.js.","ref":"architecture.html#design-trade-offs"},{"type":"extras","title":"When to Use LiveVue - How LiveVue Works","doc":"LiveVue is a good fit for complex client-side interactions, rich UI components with local state, leveraging the Vue ecosystem (animations, charts, etc.), and teams with Vue.js expertise.\n\nConsider alternatives for simple forms and basic interactions, applications prioritizing minimal JavaScript, or teams without Vue.js experience.","ref":"architecture.html#when-to-use-livevue"},{"type":"extras","title":"Next Steps - How LiveVue Works","doc":"Now that you understand how LiveVue works:\n\n- [Configuration](configuration.html) to customize behavior and SSR settings\n- [Basic Usage](basic_usage.html) for practical patterns and examples\n- [Client-Side API](client_api.html) for detailed API reference","ref":"architecture.html#next-steps"},{"type":"extras","title":"Testing","doc":"# Testing Guide\n\nLiveVue provides a robust testing module `LiveVue.Test` that makes it easy to test Vue components within your Phoenix LiveView tests.","ref":"testing.html"},{"type":"extras","title":"Overview - Testing","doc":"Testing LiveVue components differs from traditional Phoenix LiveView testing in a key way:\n- Traditional LiveView testing uses `render_component/2` to get final HTML\n- LiveVue testing provides helpers to inspect the Vue component configuration before client-side rendering","ref":"testing.html#overview"},{"type":"extras","title":"Testing Configuration - Testing","doc":"For comprehensive testing, you should disable props diffing in your test environment to ensure `LiveVue.Test.get_vue/2` always returns complete props data:\n\n```elixir\n# config/test.exs\nconfig :live_vue,\n  enable_props_diff: false\n```\n\nWhen props diffing is enabled (the default), LiveVue only sends changed properties to optimize performance. However, during testing, you typically want to inspect the complete component state rather than just the incremental changes. Disabling diffing ensures that all props are always available for testing assertions.\n\nThis configuration should be set globally for the test environment rather than per-component, as it affects the behavior of the testing helpers.","ref":"testing.html#testing-configuration"},{"type":"extras","title":"Basic Component Testing - Testing","doc":"Let's start with a simple component test:\n\n```elixir\ndefmodule MyAppWeb.CounterTest do\n  use MyAppWeb.ConnCase\n  alias LiveVue.Test\n\n  test \"renders counter component with initial props\", %{conn: conn} do\n    {:ok, view, _html} = live(conn, \"/counter\")\n    vue = Test.get_vue(view)\n\n    assert vue.component == \"Counter\"\n    assert vue.props == %{\"count\" => 0}\n  end\nend\n```\n\nThe `get_vue/2` function returns a map containing:\n- `:component` - Vue component name\n- `:id` - Unique component identifier\n- `:props` - Decoded props\n- `:handlers` - Event handlers and operations\n- `:slots` - Slot content\n- `:ssr` - SSR status\n- `:class` - CSS classes","ref":"testing.html#basic-component-testing"},{"type":"extras","title":"Testing Multiple Components - Testing","doc":"When your view contains multiple Vue components, you can specify which one to test:\n\n```elixir\n# Find by component name\nvue = Test.get_vue(view, name: \"UserProfile\")\n\n# Find by ID\nvue = Test.get_vue(view, id: \"profile-1\")\n```\n\nExample with multiple components:\n\n```elixir\ndef render(assigns) do\n  ~H\"\"\"\n   \n    <.vue id=\"profile-1\" name=\"John\" v-component=\"UserProfile\" />\n    <.vue id=\"card-1\" name=\"Jane\" v-component=\"UserCard\" />\n   \n  \"\"\"\nend\n\ntest \"finds specific component\" do\n  html = render_component(&my_component/1)\n\n  # Get UserCard component\n  vue = Test.get_vue(html, name: \"UserCard\")\n  assert vue.props == %{\"name\" => \"Jane\"}\n\n  # Get by ID\n  vue = Test.get_vue(html, id: \"profile-1\")\n  assert vue.component == \"UserProfile\"\nend\n```","ref":"testing.html#testing-multiple-components"},{"type":"extras","title":"Testing Event Handlers - Testing","doc":"You can verify event handlers are properly configured:\n\n```elixir\ntest \"component has correct event handlers\" do\n  vue = Test.get_vue(render_component(&my_component/1))\n\n  assert vue.handlers == %{\n    \"click\" => JS.push(\"click\", value: %{\"abc\" => \"def\"}),\n    \"submit\" => JS.push(\"submit\")\n  }\nend\n```","ref":"testing.html#testing-event-handlers"},{"type":"extras","title":"Testing Slots - Testing","doc":"LiveVue provides tools to test both default and named slots:\n\n```elixir\ndef component_with_slots(assigns) do\n  ~H\"\"\"\n  <.vue v-component=\"WithSlots\">\n    Default content\n    <:header>Header content</:header>\n    <:footer>Footer content</:footer>\n  </.vue>\n  \"\"\"\nend\n\ntest \"renders slots correctly\" do\n  vue = Test.get_vue(render_component(&component_with_slots/1))\n\n  assert vue.slots == %{\n    \"default\" => \"Default content\",\n    \"header\" => \"Header content\",\n    \"footer\" => \"Footer content\"\n  }\nend\n```\n\nImportant notes about slots:\n- Use `<:inner_block>` instead of `<:default>` for default content\n- Slots are automatically Base64 encoded in the HTML\n- The test helper decodes them for easier assertions","ref":"testing.html#testing-slots"},{"type":"extras","title":"Testing SSR Configuration - Testing","doc":"Verify Server-Side Rendering settings:\n\n```elixir\ntest \"respects SSR configuration\" do\n  vue = Test.get_vue(render_component(&my_component/1))\n  assert vue.ssr == true\n\n  # Or with SSR disabled\n  vue = Test.get_vue(render_component(&ssr_disabled_component/1))\n  assert vue.ssr == false\nend\n```","ref":"testing.html#testing-ssr-configuration"},{"type":"extras","title":"Testing CSS Classes - Testing","doc":"Check applied styling:\n\n```elixir\ntest \"applies correct CSS classes\" do\n  vue = Test.get_vue(render_component(&my_component/1))\n  assert vue.class == \"bg-blue-500 rounded\"\nend\n```","ref":"testing.html#testing-css-classes"},{"type":"extras","title":"Integration Testing - Testing","doc":"For full integration tests, you should use headless browser to render components. Currently this guide is a work in progress.","ref":"testing.html#integration-testing"},{"type":"extras","title":"Best Practices - Testing","doc":"1. **Component Isolation**\n   - Test Vue components in isolation when possible\n   - Use `render_component/1` for focused tests\n\n2. **Clear Assertions**\n   - Test one aspect per test\n   - Use descriptive test names\n   - Assert specific properties rather than entire component structure\n\n3. **Integration Testing**\n   - Test full component interaction in LiveView context\n   - Verify both server and client-side behavior\n   - Test error cases and edge conditions\n\n4. **Maintainable Tests**\n   - Use helper functions for common assertions\n   - Keep test setup minimal and clear\n   - Document complex test scenarios","ref":"testing.html#best-practices"},{"type":"extras","title":"Deployment","doc":"# Deployment\n\nDeploying a LiveVue app is similar to deploying a regular Phoenix app, with one key requirement: **Node.js version 19 or later must be installed** in your production environment.\n\n> #### SSR Configuration {: .tip}\n>\n> For detailed SSR configuration options, see [Configuration](configuration.html#server-side-rendering-ssr). This guide focuses on deployment-specific setup.","ref":"deployment.html"},{"type":"extras","title":"General Requirements - Deployment","doc":"1. Node.js 19+ installed in production\n2. Standard Phoenix deployment requirements\n3. Build assets before deployment","ref":"deployment.html#general-requirements"},{"type":"extras","title":"Fly.io Deployment Guide - Deployment","doc":"Here's a detailed guide for deploying to [Fly.io](https://fly.io/). Similar principles apply to other hosting providers.","ref":"deployment.html#fly-io-deployment-guide"},{"type":"extras","title":"1. Generate Dockerfile - Deployment","doc":"First, generate a Phoenix release Dockerfile:\n\n```bash\nmix phx.gen.release --docker\n```","ref":"deployment.html#1-generate-dockerfile"},{"type":"extras","title":"2. Modify Dockerfile - Deployment","doc":"Update the generated Dockerfile to include Node.js:\n\n```dockerfile\n# Build Stage\nFROM hexpm/elixir:1.14.4-erlang-25.3.2-debian-bullseye-20230227-slim AS builder\n\n# Set environment variables\n...(about 15 lines omitted)...\n\n# Install build dependencies\nRUN apt-get update -y && apt-get install -y build-essential git curl \\\n    && apt-get clean && rm -f /var/lib/apt/lists/*_*\n\n# Install Node.js for build stage\nRUN curl -fsSL https://deb.nodesource.com/setup_19.x | bash - && apt-get install -y nodejs\n\n# Copy application code\nCOPY . .\n\n# Install npm dependencies\nRUN cd /app/assets && npm install\n\n...(about 20 lines omitted)...\n\n# Production Stage\nFROM ${RUNNER_IMAGE}\n\nRUN apt-get update -y && \\\n    apt-get install -y libstdc++6 openssl libncurses5 locales ca-certificates curl \\\n    && apt-get clean && rm -f /var/lib/apt/lists/*_*\n\n# Install Node.js for production\nRUN curl -fsSL https://deb.nodesource.com/setup_19.x | bash - && apt-get install -y nodejs\n\n...(remaining dockerfile content)...\n```\n\nKey changes:\n- Add `curl` to build dependencies\n- Install Node.js in both build and production stages\n- Add npm install step for assets","ref":"deployment.html#2-modify-dockerfile"},{"type":"extras","title":"3. Launch on Fly.io - Deployment","doc":"1. Initialize your app:\n```bash\nfly launch\n```\n\n2. Configure database when prompted:\n```bash\n? Do you want to tweak these settings before proceeding? (y/N) y\n```\n\n3. In the configuration window:\n   - Choose \"Fly Postgres\" for database\n   - Name your database\n   - Consider development configuration for cost savings\n   - Review other settings as needed\n\n4. After deployment completes, open your app:\n```bash\nfly apps open\n```","ref":"deployment.html#3-launch-on-fly-io"},{"type":"extras","title":"Other Deployment Options - Deployment","doc":"","ref":"deployment.html#other-deployment-options"},{"type":"extras","title":"Heroku - Deployment","doc":"For Heroku deployment:\n1. Use the [Phoenix buildpack](https://hexdocs.pm/phoenix/heroku.html)\n2. Add Node.js buildpack:\n```bash\nheroku buildpacks:add --index 1 heroku/nodejs\n```","ref":"deployment.html#heroku"},{"type":"extras","title":"Docker - Deployment","doc":"If using your own Docker setup:\n1. Ensure Node.js 19+ is installed\n2. Follow standard Phoenix deployment practices\n3. Configure SSR for production (see [Configuration](configuration.html#production-ssr-setup))","ref":"deployment.html#docker"},{"type":"extras","title":"Custom Server - Deployment","doc":"For bare metal or VM deployments:\n1. Install Node.js 19+:\n```bash\ncurl -fsSL https://deb.nodesource.com/setup_19.x | bash -\napt-get install -y nodejs\n```\n\n2. Follow standard [Phoenix deployment guide](https://hexdocs.pm/phoenix/deployment.html)","ref":"deployment.html#custom-server"},{"type":"extras","title":"Production Checklist - Deployment","doc":"- [ ] Node.js 19+ installed\n- [ ] Assets built (`mix assets.build`)\n- [ ] SSR configured properly (see [Configuration](configuration.html#production-ssr-setup))\n- [ ] Database configured\n- [ ] Environment variables set\n- [ ] SSL certificates configured (if needed)\n- [ ] Production secrets generated\n- [ ] Release configuration tested","ref":"deployment.html#production-checklist"},{"type":"extras","title":"Troubleshooting - Deployment","doc":"","ref":"deployment.html#troubleshooting"},{"type":"extras","title":"Common Issues - Deployment","doc":"1. **SSR Not Working**\n   - Verify Node.js installation\n   - Check SSR configuration (see [Configuration](configuration.html#ssr-troubleshooting))\n   - Ensure server bundle exists in `priv/vue/server.js`\n\n2. **Asset Loading Issues**\n   - Verify assets were built\n   - Check digest configuration\n   - Inspect network requests\n\n3. **Performance Issues**\n   - Consider adjusting NodeJS pool size (see [Configuration](configuration.html#production-ssr-setup))","ref":"deployment.html#common-issues"},{"type":"extras","title":"Next Steps - Deployment","doc":"- Review [FAQ](faq.html) for common questions\n- Join our [GitHub Discussions](https://github.com/Valian/live_vue/discussions) for help\n- Consider contributing to [LiveVue](https://github.com/Valian/live_vue)","ref":"deployment.html#next-steps"},{"type":"extras","title":"FAQ","doc":"# Frequently Asked Questions","ref":"faq.html"},{"type":"extras","title":"General Questions - FAQ","doc":"","ref":"faq.html#general-questions"},{"type":"extras","title":"Why LiveVue? - FAQ","doc":"Phoenix LiveView makes it possible to create rich, interactive web apps without writing JS. However, when you need complex client-side functionality, you might end up writing lots of imperative, hard-to-maintain hooks.\n\nLiveVue allows you to create hybrid apps where:\n- Server maintains the session state\n- Vue handles complex client-side interactions\n- Both sides communicate seamlessly\n\nCommon use cases:\n- Your hooks are starting to look like jQuery\n- You have complex local state to manage\n- You want to use the Vue ecosystem (transitions, graphs, etc.)\n- You need advanced client-side features\n- You simply like Vue 😉","ref":"faq.html#why-livevue"},{"type":"extras","title":"What's with the Name? - FAQ","doc":"Yes, \"LiveVue\" sounds exactly like \"LiveView\" - we noticed slightly too late to change! Some helpful Reddit users pointed it out 😉\n\nWe suggest referring to it as \"LiveVuejs\" in speech to avoid confusion.","ref":"faq.html#what-s-with-the-name"},{"type":"extras","title":"Technical Details - FAQ","doc":"","ref":"faq.html#technical-details"},{"type":"extras","title":"How Does LiveVue Work? - FAQ","doc":"The implementation is straightforward:\n\n1. **Rendering**: Phoenix [renders](https://github.com/Valian/live_vue/blob/main/lib/live_vue.ex) a `div` with:\n   - Props as data attributes\n   - Slots as child elements\n   - Event handlers configured\n   - SSR content (when enabled)\n\n2. **Initialization**: The [LiveVue hook](https://github.com/Valian/live_vue/blob/main/assets/js/live_vue/hooks.js):\n   - Mounts on element creation\n   - Sets up event handlers\n   - Injects the hook for `useLiveVue`\n   - Mounts the Vue component\n\n3. **Updates**:\n   - Phoenix updates only changed data attributes\n   - Hook updates component props accordingly\n\n4. **Cleanup**:\n   - Vue component unmounts on destroy\n   - Garbage collection handles cleanup\n\nNote: Hooks fire only after `app.js` loads, which may cause slight delays in initial render.\n\nFor a deeper dive into the architecture, see [Architecture](architecture.html).","ref":"faq.html#how-does-livevue-work"},{"type":"extras","title":"What Optimizations Does LiveVue Use? - FAQ","doc":"LiveVue implements several performance optimizations:\n\n1. **Selective Updates**:\n   - Only changed props/handlers/slots are sent to client\n   - Achieved through careful `__changed__` assign modifications\n\n2. **Efficient Props Handling**:\n   ```elixir\n   data-props={\"#{@props |> Jason.encode()}\"}\n   ```\n   String interpolation prevents sending `data-props=` on each update\n\n3. **Struct Encoding and Diffing**:\n   - Uses `LiveVue.Encoder` protocol to convert structs to maps\n   - Enables efficient JSON patch calculations (using [Jsonpatch](https://github.com/corka149/jsonpatch) library)\n   - Reduces payload sizes by sending only changed fields\n\n4. **Coming Soon**:\n   - Sending only updated props","ref":"faq.html#what-optimizations-does-livevue-use"},{"type":"extras","title":"What is the LiveVue.Encoder Protocol? - FAQ","doc":"The `LiveVue.Encoder` protocol is a crucial part of LiveVue's architecture that safely converts Elixir structs to maps before JSON serialization. It serves several important purposes:\n\n**Why it's needed:**\n- **Security**: Prevents accidental exposure of sensitive struct fields\n- **Performance**: Enables efficient JSON patch diffing by providing consistent data structures\n- **Explicit Control**: Forces developers to be intentional about what data is sent to the client\n\n**How to use it:**\n```elixir\ndefmodule User do\n  @derive LiveVue.Encoder\n  defstruct [:name, :email, :age]\nend\n```\n\nFor complete implementation details including field selection, custom implementations, and third-party structs, see [Component Reference](component_reference.html#custom-structs-with-livevue-encoder).\n\nWithout implementing this protocol, you'll get a `Protocol.UndefinedError` when trying to pass custom structs as props. This is by design - it's a safety feature to prevent accidental data exposure.\n\nThe protocol is similar to `Jason.Encoder` but converts structs to maps instead of JSON strings, which allows LiveVue to calculate minimal diffs and send only changed data over WebSocket connections.","ref":"faq.html#what-is-the-livevue-encoder-protocol"},{"type":"extras","title":"Why is SSR Useful? - FAQ","doc":"SSR (Server-Side Rendering) provides several benefits:\n\n1. **Initial Render**: Components appear immediately, before JS loads\n2. **SEO**: Search engines see complete content\n3. **Performance**: Reduces client-side computation\n\nImportant notes:\n- SSR runs only during \"dead\" renders (no socket)\n- Not needed during live navigation\n- Can be disabled per-component with `v-ssr={false}`\n\nFor complete SSR configuration, see [Configuration](configuration.html#server-side-rendering-ssr).","ref":"faq.html#why-is-ssr-useful"},{"type":"extras","title":"Can I nest LiveVue components inside each other? - FAQ","doc":"No, it is not possible to nest a `<.vue>` component rendered by LiveView inside another `<.vue>` component's slot.\n\n**Why?**\n\nThis limitation exists because of how slots are handled. The content you place inside a component's slot in your `.heex` template is rendered into raw HTML on the server *before* being sent to the client. When the parent Vue component is mounted on the client, it receives this HTML as a simple string.\n\nSince the nested component's HTML is just inert markup at that point, Phoenix LiveView's hooks (including the `VueHook` that powers LiveVue) cannot be attached to it, and the nested Vue component will never be initialized.\n\n**Workarounds:**\n\n1.  **Adjacent Components:** The simplest approach is to restructure your UI to use adjacent components instead of nested ones.\n\n    ```elixir\n    # Instead of this:\n    <.Card v-socket={@socket}>\n      <.UserProfile user={@user} v-socket={@socket} />\n    </.Card>\n\n    # Do this:\n    <.Card v-socket={@socket} />\n    <.UserProfile user={@user} v-socket={@socket} />\n    ```\n\n2.  **Standard Vue Components:** You can nest standard (non-LiveVue) Vue components inside a LiveVue component. These child components are defined entirely within the parent's Vue template and do not have a corresponding `<.vue>` tag in LiveView. They can receive props from their LiveVue parent and manage their own state as usual.\n\n    ```html\n    <!-- Parent: MyLiveVueComponent.vue -->\n     \n    import StandardChild from './StandardChild.vue';\n    defineProps(['someData']);\n     \n     \n       \n         Data from LiveView: {{ someData }} \n         \n       \n     \n    ```","ref":"faq.html#can-i-nest-livevue-components-inside-each-other"},{"type":"extras","title":"Development - FAQ","doc":"","ref":"faq.html#development"},{"type":"extras","title":"How Do I Use TypeScript? - FAQ","doc":"LiveVue provides full TypeScript support:\n\n1. Use the example `tsconfig.json`\n2. Check `example_project/assets/ts_config_example` for:\n   - LiveVue entrypoint\n   - Tailwind setup\n   - Vite config\n\nFor `app.js`, since it's harder to convert directly:\n```javascript\n// Write your code in TypeScript\n// app.ts\nexport const initApp = () => { /* ... */ }\n\n// Import in app.js\nimport {initApp} from './app.ts'\ninitApp()\n```\n\nFor complete TypeScript setup, see [Configuration](configuration.html#typescript-support).","ref":"faq.html#how-do-i-use-typescript"},{"type":"extras","title":"Where Should I Put Vue Files? - FAQ","doc":"Vue files in LiveVue are similar to HEEX templates. You have two main options:\n\n1. **Default Location**: `assets/vue` directory\n2. **Colocated**: Next to your LiveViews in `lib/my_app_web`\n\nColocating provides better DX by:\n- Keeping related code together\n- Making relationships clearer\n- Simplifying maintenance\n\nNo configuration needed - just place `.vue` files in `lib/my_app_web` and reference them by name or path.\n\nFor advanced component organization, see [Configuration](configuration.html#component-organization).","ref":"faq.html#where-should-i-put-vue-files"},{"type":"extras","title":"Comparison with Other Solutions - FAQ","doc":"","ref":"faq.html#comparison-with-other-solutions"},{"type":"extras","title":"How Does LiveVue Compare to LiveSvelte? - FAQ","doc":"Both serve similar purposes with similar implementations, but have key differences:\n\n**Technical Differences**:\n- Vue uses virtual DOM, Svelte doesn't\n- Vue bundle is slightly larger due to runtime\n- Performance is comparable\n\n**Reactivity Approach**:\n- Svelte: Compilation-based, concise but with [some limitations](https://thoughtspile.github.io/2023/04/22/svelte-state/)\n- Vue: Proxy-based, more verbose but more predictable\n\n**Future Developments**:\n- Vue is working on [Vapor mode](https://github.com/vuejs/core-vapor) (no virtual DOM)\n- Svelte 5 Runes will be similar to Vue `ref`\n\n**Ecosystem**:\n- Vue has a larger ecosystem\n- More third-party components available\n- Larger community\n\nChoose based on:\n- Your team's familiarity\n- Ecosystem requirements\n- Syntax preferences\n- Bundle size concerns\n\nFor a detailed comparison with other solutions, see [Comparison](comparison.html).","ref":"faq.html#how-does-livevue-compare-to-livesvelte"},{"type":"extras","title":"Additional Resources - FAQ","doc":"- [GitHub Discussions](https://github.com/Valian/live_vue/discussions)\n- [Example Project](https://github.com/Valian/live_vue/tree/main/example_project)\n- [Vue Documentation](https://vuejs.org/)\n- [Phoenix LiveView Documentation](https://hexdocs.pm/phoenix_live_view)","ref":"faq.html#additional-resources"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting Guide\n\nThis guide helps you diagnose and fix common issues when working with LiveVue.\n\n> #### Quick Start {: .tip}\n>\n> New to LiveVue? Start with [Getting Started](getting_started.html) for a working example, then check [Basic Usage](basic_usage.html) for common patterns.","ref":"troubleshooting.html"},{"type":"extras","title":"Component Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#component-issues"},{"type":"extras","title":"Component Not Rendering - Troubleshooting","doc":"**Symptoms:**\n- Empty div where component should be\n- No errors in console\n- Component works in isolation\n\n**Possible Causes & Solutions:**\n\n1. **Missing `v-socket` attribute**\n   ```elixir\n   # ❌ Missing v-socket\n   <.vue v-component=\"Counter\" count={@count} />\n\n   # ✅ Correct\n   <.vue v-component=\"Counter\" count={@count} v-socket={@socket} />\n   ```\n\n2. **Component name mismatch**\n   ```elixir\n   # ❌ File: Counter.vue, but using wrong name\n   <.vue v-component=\"counter\" v-socket={@socket} />\n\n   # ✅ Correct - case sensitive\n   <.vue v-component=\"Counter\" v-socket={@socket} />\n   ```\n\n   Check browser console for errors.\n\n3. **Component not found in resolution**\n   ```javascript\n   // Check your component resolution in assets/vue/index.js\n   const components = {\n     ...import.meta.glob(\"./**/*.vue\", { eager: true }),\n   }\n\n   // Debug: log available components\n   console.log(\"Available components:\", Object.keys(components))\n   ```","ref":"troubleshooting.html#component-not-rendering"},{"type":"extras","title":"Component Renders But Doesn't Update - Troubleshooting","doc":"**Symptoms:**\n- Component shows initial state\n- Props don't update when server state changes\n- No reactivity\n\n**Solutions:**\n\n1. **Check prop names match**\n   ```elixir\n   # Server side\n   <.vue user_name={@user.name} v-component=\"Profile\" v-socket={@socket} />\n   ```\n\n   ```html\n   <!-- Client side - prop names must match exactly -->\n    \n   const props = defineProps<{\n     user_name: string  // Must match server prop name\n   }>()\n    \n   ```\n\n2. **Verify assigns are updating**\n   ```elixir\n   # Add debug logging\n   def handle_event(\"update\", _params, socket) do\n     IO.inspect(socket.assigns, label: \"Before update\")\n     socket = assign(socket, :count, socket.assigns.count + 1)\n     IO.inspect(socket.assigns, label: \"After update\")\n     {:noreply, socket}\n   end\n   ```\n\n3. **Use Vue.js devtools to inspect the component**\n\n   Open Vue.js devtools in browser and inspect the component.\n   Check if the component is receiving the correct props.\n   Check if the component is re-rendering when the props change.","ref":"troubleshooting.html#component-renders-but-doesn-t-update"},{"type":"extras","title":"LiveVue.Encoder Protocol Issues - Troubleshooting","doc":"**Symptoms:**\n- `Protocol.UndefinedError` when passing structs as props\n- Component doesn't render with custom struct props\n- Error mentions \"LiveVue.Encoder protocol must always be explicitly implemented\"\n\n**Solutions:**\n\n1. **Implement the encoder protocol for your structs**\n   ```elixir\n   defmodule User do\n     @derive LiveVue.Encoder\n     defstruct [:name, :email, :age]\n   end\n   ```\n\n2. **For third-party structs, use Protocol.derive/3**\n   ```elixir\n   # In your application.ex or relevant module\n   Protocol.derive(LiveVue.Encoder, SomeLibrary.User, only: [:id, :name])\n   ```\n\n3. **Debug encoder issues**\n   ```elixir\n   # Test your encoder implementation\n   iex> user = %User{name: \"John\", email: \"john@example.com\"}\n   iex> LiveVue.Encoder.encode(user)\n   %{name: \"John\", email: \"john@example.com\"}\n   ```\n\nFor complete implementation details including field selection and custom implementations, see [Component Reference](component_reference.html#custom-structs-with-livevue-encoder).\n\n**Common Causes:**\n- Passing structs without implementing the encoder protocol\n- Nested structs where some don't have the protocol implemented\n- Third-party library structs that need protocol derivation","ref":"troubleshooting.html#livevue-encoder-protocol-issues"},{"type":"extras","title":"TypeScript Errors - Troubleshooting","doc":"**Common Error: `Cannot find module 'live_vue'`**\n\n```typescript\n// Add to your env.d.ts or types.d.ts\ndeclare module 'live_vue' {\n  export function useLiveVue(): any\n  export function createLiveVue(config: any): any\n  export function findComponent(components: any, name: string): any\n  export function getHooks(app: any): any\n}\n```\n\n**Error: `Property 'xxx' does not exist on type`**\n\n```typescript\n// Define proper interfaces\ninterface Props {\n  count: number\n  user: {\n    id: number\n    name: string\n  }\n}\n\nconst props = defineProps ()\n```","ref":"troubleshooting.html#typescript-errors"},{"type":"extras","title":"Event Handling Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#event-handling-issues"},{"type":"extras","title":"Events Not Firing - Troubleshooting","doc":"**Symptoms:**\n- Clicking buttons does nothing\n- No events reach LiveView\n- Console shows no errors\n\n**Solutions:**\n\n1. **Check event handler syntax**\n   ```elixir\n   # ❌ Wrong syntax\n   <.vue v-on-click={JS.push(\"increment\")} />\n\n   # ✅ Correct syntax\n   <.vue v-on:click={JS.push(\"increment\")} />\n   ```\n\n2. **Verify event names match**\n   ```html\n   <!-- Vue component -->\n    +1 \n   ```\n\n   ```elixir\n   <!-- LiveView template -->\n   <.vue v-on:increment={JS.push(\"inc\")} />\n\n   def handle_event(\"inc\", %{\"amount\" => amount}, socket) do\n     # Handle event\n   end\n   ```\n\n3. **Check payload structure**\n   ```elixir\n   # Debug event payload\n   def handle_event(\"save\", params, socket) do\n     IO.inspect(params, label: \"Event params\")\n     {:noreply, socket}\n   end\n   ```","ref":"troubleshooting.html#events-not-firing"},{"type":"extras","title":"Events Fire But Handler Not Called - Troubleshooting","doc":"**Check handler function exists:**\n```elixir\n# Make sure you have the handler defined\ndef handle_event(\"my_event\", _params, socket) do\n  {:noreply, socket}\nend\n```\n\n**Verify event name spelling:**\n```elixir\n# Event names are case-sensitive\n<.vue v-on:save-user={JS.push(\"save_user\")} />  # save-user → save_user\n```","ref":"troubleshooting.html#events-fire-but-handler-not-called"},{"type":"extras","title":"Build and Development Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#build-and-development-issues"},{"type":"extras","title":"Vite Server Not Starting - Troubleshooting","doc":"**Error: `EADDRINUSE: address already in use`**\n\n```bash\n# Kill process using port 5173\nlsof -ti:5173 | xargs kill -9\n\n# Or use different port\nnpm run dev -- --port 5174\n```\n\n**Error: `Module not found`**\n\n```bash\n# Clear node_modules and reinstall\nrm -rf node_modules package-lock.json\nnpm install\n```","ref":"troubleshooting.html#vite-server-not-starting"},{"type":"extras","title":"Build Failures - Troubleshooting","doc":"**TypeScript compilation errors:**\n\n```bash\n# Check TypeScript version compatibility\nnpm install typescript@5.5.4 vue-tsc@2.10.0\n```\n\n**Vite build errors:**\n\n```bash\n# Clear Vite cache\nrm -rf node_modules/.vite\nnpm run build\n```","ref":"troubleshooting.html#build-failures"},{"type":"extras","title":"Hot Reload Not Working - Troubleshooting","doc":"1. **Check Vite configuration:**\n   ```javascript\n   // vite.config.js\n   export default defineConfig({\n     server: {\n       host: '0.0.0.0',  // Allow external connections\n       port: 5173,\n       hmr: true\n     }\n   })\n   ```\n\n2. **Verify watcher configuration:**\n   ```elixir\n   # config/dev.exs\n   config :my_app, MyAppWeb.Endpoint,\n     watchers: [\n       npm: [\"--silent\", \"run\", \"dev\", cd: Path.expand(\"../assets\", __DIR__)]\n     ]\n   ```","ref":"troubleshooting.html#hot-reload-not-working"},{"type":"extras","title":"SSR Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#ssr-issues"},{"type":"extras","title":"SSR Not Working - Troubleshooting","doc":"**Check SSR configuration:**\n```elixir\n# config/dev.exs\nconfig :live_vue,\n  ssr_module: LiveVue.SSR.ViteJS,\n  vite_host: \"http://localhost:5173\",\n  ssr: true\n```\n\nFor complete SSR configuration options, see [Configuration](configuration.html#server-side-rendering-ssr).\n\n**Verify Node.js version:**\n```bash\nnode --version  # Should be 19+\n```","ref":"troubleshooting.html#ssr-not-working"},{"type":"extras","title":"SSR Errors in Production - Troubleshooting","doc":"**Check NodeJS supervisor:**\n```elixir\n# application.ex\nchildren = [\n  {NodeJS.Supervisor, [path: LiveVue.SSR.NodeJS.server_path(), pool_size: 4]},\n  # ... other children\n]\n```\n\n**Verify server bundle exists:**\n```bash\nls priv/vue/server.js  # Should exist after build\n```\n\nFor production SSR setup details, see [Configuration](configuration.html#production-ssr-setup).","ref":"troubleshooting.html#ssr-errors-in-production"},{"type":"extras","title":"Performance Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#performance-issues"},{"type":"extras","title":"Slow Initial Load - Troubleshooting","doc":"1. **Enable lazy loading:**\n   ```javascript\n   // assets/vue/index.js\n   const components = {\n     Counter: () => import('./Counter.vue'),\n     Modal: () => import('./Modal.vue')\n   }\n   ```\n\n2. **Optimize bundle size:**\n   ```bash\n   # Analyze bundle\n   npm run build -- --analyze\n   ```","ref":"troubleshooting.html#slow-initial-load"},{"type":"extras","title":"Memory Leaks - Troubleshooting","doc":"**Clean up event listeners:**\n```html\n \nimport { onUnmounted } from 'vue'\nimport { useLiveVue } from 'live_vue'\n\nconst live = useLiveVue()\n\nconst cleanup = live.handleEvent('data_update', handleUpdate)\n\nonUnmounted(() => {\n  cleanup()  // Important: clean up listeners\n})\n \n```\n\n**Clear timers and intervals:**\n```html\n \nimport { onUnmounted } from 'vue'\n\nconst interval = setInterval(() => {\n  // Do something\n}, 1000)\n\nonUnmounted(() => {\n  clearInterval(interval)\n})\n \n```","ref":"troubleshooting.html#memory-leaks"},{"type":"extras","title":"Debugging Techniques - Troubleshooting","doc":"","ref":"troubleshooting.html#debugging-techniques"},{"type":"extras","title":"Enable Debug Mode - Troubleshooting","doc":"```javascript\n// In browser console or app.js\nwindow.liveVueDebug = true\n```","ref":"troubleshooting.html#enable-debug-mode"},{"type":"extras","title":"Component Inspection - Troubleshooting","doc":"1. **Use Vue DevTools browser extension**\n2. **Add debug logging:**\n   ```html\n    \n   import { watch } from 'vue'\n\n   const props = defineProps<{count: number}>()\n\n   watch(() => props.count, (newVal, oldVal) => {\n     console.log('Count changed:', oldVal, '→', newVal)\n   })\n    \n   ```","ref":"troubleshooting.html#component-inspection"},{"type":"extras","title":"Network Debugging - Troubleshooting","doc":"1. **Monitor WebSocket traffic in browser DevTools**\n2. **Log LiveView events:**\n   ```elixir\n   def handle_event(event, params, socket) do\n     IO.inspect({event, params}, label: \"LiveView Event\")\n     # ... handle event\n   end\n   ```","ref":"troubleshooting.html#network-debugging"},{"type":"extras","title":"Common Error Messages - Troubleshooting","doc":"### `Cannot read property 'mount' of undefined`\n\n**Cause:** Component resolution failed\n\n**Solution:** Check component name and file path\n```javascript\n// Debug component resolution\n// find component does it by default, you might need to do it if you override it\nconsole.log(\"Resolving component:\", componentName)\nconsole.log(\"Available components:\", Object.keys(components))\n```\n\n### `ReferenceError: process is not defined`\n\n**Cause:** Node.js globals in browser code\n\n**Solution:** Add to Vite config:\n```javascript\n// vite.config.js\nexport default defineConfig({\n  define: {\n    global: 'globalThis',\n    'process.env': {}\n  }\n})\n```\n\n### `Module \"live_vue\" has been externalized`\n\n**Cause:** SSR configuration issue\n\n**Solution:** Check Vite SSR config:\n```javascript\n// vite.config.js\nexport default defineConfig({\n  ssr: {\n    noExternal: ['live_vue']\n  }\n})\n```","ref":"troubleshooting.html#common-error-messages"},{"type":"extras","title":"Getting Help - Troubleshooting","doc":"","ref":"troubleshooting.html#getting-help"},{"type":"extras","title":"Before Asking for Help - Troubleshooting","doc":"1. **Check browser console for errors**\n2. **Verify all configuration steps** (see [Configuration](configuration.html))\n3. **Test with minimal reproduction case**\n4. **Check if issue exists in example project**","ref":"troubleshooting.html#before-asking-for-help"},{"type":"extras","title":"Where to Get Help - Troubleshooting","doc":"1. **GitHub Issues**: For bugs and feature requests\n2. **GitHub Discussions**: For questions and community help\n3. **Elixir Forum**: For general Phoenix/Elixir questions\n4. **Vue.js Discord**: For Vue-specific questions","ref":"troubleshooting.html#where-to-get-help"},{"type":"extras","title":"Creating Bug Reports - Troubleshooting","doc":"Include:\n1. **LiveVue version**\n2. **Phoenix/LiveView versions**\n3. **Node.js version**\n4. **Minimal reproduction case**\n5. **Error messages and stack traces**\n6. **Browser and OS information**","ref":"troubleshooting.html#creating-bug-reports"},{"type":"extras","title":"Next Steps - Troubleshooting","doc":"- [FAQ](faq.html) for conceptual questions\n- [Architecture](architecture.html) to understand how things work\n- [Configuration](configuration.html) for advanced setup options\n- [GitHub Issues](https://github.com/Valian/live_vue/issues) to report bugs","ref":"troubleshooting.html#next-steps"},{"type":"extras","title":"LiveVue vs Alternatives","doc":"# LiveVue vs Alternatives\n\nThis guide compares LiveVue with other Phoenix LiveView frontend integration libraries to help you choose the right solution for your project.","ref":"comparison.html"},{"type":"extras","title":"Overview - LiveVue vs Alternatives","doc":"LiveVue is part of a growing ecosystem of libraries that integrate modern frontend frameworks with Phoenix LiveView. Each library takes a different approach to solving the challenge of combining server-side state management with rich client-side interactivity.","ref":"comparison.html#overview"},{"type":"extras","title":"Comparison Matrix - LiveVue vs Alternatives","doc":"| Feature | LiveVue | LiveSvelte | LiveReact | LiveState |\n|---------|---------|------------|-----------|-----------|\n| **Framework** | Vue.js | Svelte | React | Framework-agnostic |\n| **SSR Support** | ✅ | ✅ | ✅ | ❌ |\n| **Component Shortcuts** | ✅ | ✅ | ❌ | ❌ |\n| **Slots Support** | ✅ | ✅ | ✅ | ❌ |\n| **TypeScript Support** | ✅ | ✅ | ✅ | ✅ |\n| **Build System** | Vite | Custom esbuild | Vite | Any |\n| **Sigil DSL** | ✅ (`~VUE`) | ✅ (`~V`) | ❌ | ❌ |\n| **Embeddable Apps** | ❌ | ❌ | ❌ | ✅ |\n| **Real-time Updates** | ✅ | ✅ | ✅ | ✅ |\n| **Event Handling** | ✅ | ✅ | ✅ | ✅ |\n| **Maturity** | Mature | Mature | Stable | Stable |","ref":"comparison.html#comparison-matrix"},{"type":"extras","title":"LiveSvelte - LiveVue vs Alternatives","doc":"[LiveSvelte](https://github.com/woutdp/live_svelte) integrates Svelte with Phoenix LiveView, offering end-to-end reactivity.","ref":"comparison.html#livesvelte"},{"type":"extras","title":"Key Features - LiveVue vs Alternatives","doc":"- **Sigil DSL**: Use `~V` sigil to write Svelte code directly in LiveView templates\n- **Server-Side Rendering**: Full SSR support with hydration\n- **Slot Interoperability**: Pass LiveView content to Svelte components\n- **Component Macro**: Auto-generate component functions from Svelte files\n- **Preprocessing Support**: Built-in support for TypeScript, SCSS, etc.","ref":"comparison.html#key-features"},{"type":"extras","title":"Example Usage - LiveVue vs Alternatives","doc":"```elixir\n# Component approach\n<.svelte name=\"Counter\" props={%{count: @count}} socket={@socket} />\n\n# Sigil DSL approach\ndef render(assigns) do\n  ~V\"\"\"\n   \n    export let count = 0\n    let localState = 1\n    $: combined = count + localState\n   \n\n   Server count: {count} \n   Local state: {localState} \n   Combined: {combined} \n\n   Server increment \n   Local increment \n  \"\"\"\nend\n```","ref":"comparison.html#example-usage"},{"type":"extras","title":"When to Choose LiveSvelte - LiveVue vs Alternatives","doc":"- You prefer Svelte's syntax and reactivity model\n- You want to use Svelte as an alternative LiveView DSL\n- You need advanced preprocessing (TypeScript, SCSS, etc.)\n- You want the most mature LiveView + frontend framework integration\n- You're building complex animations or transitions","ref":"comparison.html#when-to-choose-livesvelte"},{"type":"extras","title":"Considerations - LiveVue vs Alternatives","doc":"- Custom build system (not standard esbuild)\n- Larger learning curve if unfamiliar with Svelte\n- \"Secret state\" caveat (client code visible to users)","ref":"comparison.html#considerations"},{"type":"extras","title":"LiveReact - LiveVue vs Alternatives","doc":"[LiveReact](https://github.com/mrdotb/live_react) brings React components into Phoenix LiveView applications.","ref":"comparison.html#livereact"},{"type":"extras","title":"Key Features - LiveVue vs Alternatives","doc":"- **SSR Support**: Server-side rendering with hydration\n- **TypeScript Support**: Full TypeScript integration\n- **Vite Integration**: Modern build tooling\n- **Context Provider**: LiveReact context for accessing LiveView functions\n- **Inner Block Slots**: Support for passing content to React components","ref":"comparison.html#key-features-1"},{"type":"extras","title":"Example Usage - LiveVue vs Alternatives","doc":"```elixir\n<.react name=\"Counter\" props={%{count: @count}} socket={@socket} />\n```\n\n```jsx\n// Counter.jsx\nimport { useLiveReact } from 'live_react'\n\nexport default function Counter({ count }) {\n  const { pushEvent } = useLiveReact()\n\n  return (\n     \n       Count: {count} \n       \n        Increment\n       \n     \n  )\n}\n```","ref":"comparison.html#example-usage-1"},{"type":"extras","title":"When to Choose LiveReact - LiveVue vs Alternatives","doc":"- Your team has React expertise\n- You want to reuse existing React components\n- You need access to the vast React ecosystem\n- You prefer JSX syntax\n- You're migrating from a React application","ref":"comparison.html#when-to-choose-livereact"},{"type":"extras","title":"Considerations - LiveVue vs Alternatives","doc":"- Newer library (less mature than LiveSvelte)\n- React's larger bundle size\n- More complex state management patterns in React","ref":"comparison.html#considerations-1"},{"type":"extras","title":"LiveState - LiveVue vs Alternatives","doc":"[LiveState](https://github.com/launchscout/live_state) takes a different approach, focusing on embeddable web applications and framework-agnostic state management.","ref":"comparison.html#livestate"},{"type":"extras","title":"Key Features - LiveVue vs Alternatives","doc":"- **Framework Agnostic**: Works with any frontend framework or vanilla JS\n- **Embeddable Apps**: Designed for building widgets that embed in other sites\n- **Channel-Based**: Uses Phoenix Channels for real-time communication\n- **Event-Driven**: Client dispatches events, server manages state\n- **JSON Patch Updates**: Efficient state synchronization","ref":"comparison.html#key-features-2"},{"type":"extras","title":"Example Usage - LiveVue vs Alternatives","doc":"```elixir\n# Channel\ndefmodule MyApp.TodoChannel do\n  use LiveState.Channel, web_module: MyAppWeb\n\n  def init(_channel, _payload, _socket) do\n    {:ok, %{todos: []}}\n  end\n\n  def handle_event(\"add_todo\", todo, %{todos: todos}) do\n    {:noreply, %{todos: [todo | todos]}}\n  end\nend\n```\n\n```javascript\n// Client (any framework)\nimport LiveState from 'phx-live-state'\n\nconst liveState = new LiveState({\n  url: 'ws://localhost:4000/socket',\n  topic: 'todos:lobby'\n})\n\nliveState.addEventListener('state_change', (state) => {\n  // Update UI with new state\n})\n\nliveState.dispatchEvent('add_todo', { title: 'New todo' })\n```","ref":"comparison.html#example-usage-2"},{"type":"extras","title":"When to Choose LiveState - LiveVue vs Alternatives","doc":"- You're building embeddable widgets for third-party sites\n- You want framework flexibility (React, Vue, Svelte, vanilla JS)\n- You need real-time updates across multiple clients\n- You're building highly interactive components within existing apps\n- You want minimal client-side complexity","ref":"comparison.html#when-to-choose-livestate"},{"type":"extras","title":"Considerations - LiveVue vs Alternatives","doc":"- No SSR support\n- More setup required for basic use cases\n- Different mental model from traditional LiveView\n- Best suited for specific use cases (embeddable apps)","ref":"comparison.html#considerations-2"},{"type":"extras","title":"LiveVue - LiveVue vs Alternatives","doc":"LiveVue provides seamless Vue.js integration with Phoenix LiveView.","ref":"comparison.html#livevue"},{"type":"extras","title":"Key Features - LiveVue vs Alternatives","doc":"- **End-to-End Reactivity**: Automatic prop updates from LiveView\n- **Server-Side Rendering**: Optional SSR with configurable settings\n- **Component Shortcuts**: Auto-generated component functions\n- **Slot Support**: Pass LiveView content to Vue components\n- **TypeScript Support**: Full TypeScript integration with Vite\n- **Event Handling**: Multiple approaches for handling events","ref":"comparison.html#key-features-3"},{"type":"extras","title":"Example Usage - LiveVue vs Alternatives","doc":"```elixir\n<.vue\n  v-component=\"Counter\"\n  v-socket={@socket}\n  count={@count}\n  v-on:increment={JS.push(\"inc\")}\n/>\n```\n\n```html\n<!-- Counter.vue -->\n \n   \n     Count: {{ count }} \n     Increment \n   \n \n\n \ndefineProps(['count'])\ndefineEmits(['increment'])\n \n```","ref":"comparison.html#example-usage-3"},{"type":"extras","title":"When to Choose LiveVue - LiveVue vs Alternatives","doc":"- Your team has Vue.js expertise\n- You want a balance between simplicity and power\n- You prefer Vue's template syntax and reactivity system\n- You need good TypeScript support with modern tooling\n- You want comprehensive documentation and examples","ref":"comparison.html#when-to-choose-livevue"},{"type":"extras","title":"Decision Framework - LiveVue vs Alternatives","doc":"","ref":"comparison.html#decision-framework"},{"type":"extras","title":"Choose LiveVue if: - LiveVue vs Alternatives","doc":"- You're experienced with Vue.js\n- You want excellent documentation and examples\n- You need a balance of features and simplicity\n- You prefer Vue's template syntax and composition API","ref":"comparison.html#choose-livevue-if"},{"type":"extras","title":"Choose LiveSvelte if: - LiveVue vs Alternatives","doc":"- You love Svelte's syntax and reactivity model\n- You want to use frontend framework as LiveView DSL\n- You need the most mature integration\n- You're building animation-heavy applications","ref":"comparison.html#choose-livesvelte-if"},{"type":"extras","title":"Choose LiveReact if: - LiveVue vs Alternatives","doc":"- Your team is React-focused\n- You want to reuse existing React components\n- You need access to React's ecosystem\n- You're migrating from a React application","ref":"comparison.html#choose-livereact-if"},{"type":"extras","title":"Choose LiveState if: - LiveVue vs Alternatives","doc":"- You're building embeddable widgets\n- You need framework flexibility\n- You want real-time updates across clients\n- You're building highly interactive components","ref":"comparison.html#choose-livestate-if"},{"type":"extras","title":"Stick with Pure LiveView if: - LiveVue vs Alternatives","doc":"- Your app doesn't need complex client-side interactivity\n- You want to minimize JavaScript complexity\n- Your team is primarily backend-focused\n- You're building traditional web applications","ref":"comparison.html#stick-with-pure-liveview-if"},{"type":"extras","title":"Migration Considerations - LiveVue vs Alternatives","doc":"","ref":"comparison.html#migration-considerations"},{"type":"extras","title":"From LiveView to Any Integration - LiveVue vs Alternatives","doc":"1. **Identify Components**: Determine which parts need client-side interactivity\n2. **State Boundaries**: Decide what state lives on server vs client\n3. **Event Patterns**: Plan how events flow between client and server\n4. **Build Process**: Update your asset pipeline for the chosen framework","ref":"comparison.html#from-liveview-to-any-integration"},{"type":"extras","title":"Between Integrations - LiveVue vs Alternatives","doc":"Most integrations follow similar patterns, making migration feasible:\n\n1. **Component Structure**: All use similar prop-passing patterns\n2. **Event Handling**: All support bidirectional event communication\n3. **SSR**: Most support server-side rendering\n4. **Build Tools**: May require build system changes","ref":"comparison.html#between-integrations"},{"type":"extras","title":"Performance Considerations - LiveVue vs Alternatives","doc":"","ref":"comparison.html#performance-considerations"},{"type":"extras","title":"Bundle Size - LiveVue vs Alternatives","doc":"- **LiveState**: Smallest (framework-agnostic)\n- **LiveSvelte**: Small (Svelte compiles to minimal code)\n- **LiveVue**: Medium (Vue runtime)\n- **LiveReact**: Largest (React + ReactDOM)","ref":"comparison.html#bundle-size"},{"type":"extras","title":"Runtime Performance - LiveVue vs Alternatives","doc":"- **Svelte**: Fastest (compiled, no virtual DOM)\n- **Vue**: Fast (optimized virtual DOM)\n- **React**: Good (virtual DOM with optimizations)\n- **LiveState**: Depends on chosen framework","ref":"comparison.html#runtime-performance"},{"type":"extras","title":"Server Load - LiveVue vs Alternatives","doc":"All integrations have similar server load characteristics since they use LiveView's WebSocket connection for state synchronization.","ref":"comparison.html#server-load"},{"type":"extras","title":"Community and Ecosystem - LiveVue vs Alternatives","doc":"","ref":"comparison.html#community-and-ecosystem"},{"type":"extras","title":"LiveSvelte - LiveVue vs Alternatives","doc":"- **Stars**: ~1.5k GitHub stars\n- **Maturity**: Most mature integration\n- **Community**: Active development and community\n- **Documentation**: Comprehensive with examples","ref":"comparison.html#livesvelte-1"},{"type":"extras","title":"LiveVue - LiveVue vs Alternatives","doc":"- **Stars**: ~330 GitHub stars\n- **Maturity**: Stable and well-documented\n- **Community**: Growing community\n- **Documentation**: Extensive guides and examples","ref":"comparison.html#livevue-1"},{"type":"extras","title":"LiveReact - LiveVue vs Alternatives","doc":"- **Stars**: ~260 GitHub stars\n- **Maturity**: Recently reached v1.0\n- **Community**: Active development\n- **Documentation**: Good documentation","ref":"comparison.html#livereact-1"},{"type":"extras","title":"LiveState - LiveVue vs Alternatives","doc":"- **Stars**: ~260 GitHub stars\n- **Maturity**: Stable for specific use cases\n- **Community**: Focused on embeddable apps\n- **Documentation**: Good for target use cases","ref":"comparison.html#livestate-1"},{"type":"extras","title":"Conclusion - LiveVue vs Alternatives","doc":"Each library serves different needs in the Phoenix LiveView ecosystem:\n\n- **LiveVue** offers excellent Vue.js integration with comprehensive features\n- **LiveSvelte** provides the most mature integration with unique DSL capabilities\n- **LiveReact** brings React's ecosystem to LiveView applications\n- **LiveState** enables embeddable applications and framework flexibility\n\nChoose based on your team's expertise, project requirements, and long-term goals. All options provide solid foundations for building interactive web applications with Phoenix LiveView.","ref":"comparison.html#conclusion"},{"type":"extras","title":"Further Reading - LiveVue vs Alternatives","doc":"- [LiveVue Documentation](https://hexdocs.pm/live_vue)\n- [LiveSvelte Documentation](https://hexdocs.pm/live_svelte)\n- [LiveReact Documentation](https://hexdocs.pm/live_react)\n- [LiveState Documentation](https://hexdocs.pm/live_state)\n- [Phoenix LiveView Documentation](https://hexdocs.pm/phoenix_live_view)","ref":"comparison.html#further-reading"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n<!-- %% CHANGELOG_ENTRIES %% -->","ref":"changelog.html"},{"type":"extras","title":"0.6.1 - 2025-07-24 - Changelog","doc":"","ref":"changelog.html#0-6-1-2025-07-24"},{"type":"extras","title":"Fixes 🐛 - Changelog","doc":"- correctly encode props in SSR mode\n- correctly handle diffs for nil values in props\n- added package version to the package.json","ref":"changelog.html#fixes"},{"type":"extras","title":"0.6.0 - 2025-07-22 - Changelog","doc":"","ref":"changelog.html#0-6-0-2025-07-22"},{"type":"extras","title":"✨ Features and Improvements - Changelog","doc":"*   **JSON Patch Diffs for Props**: LiveVue now uses JSON Patch operations to send only the minimal differences when props change, dramatically reducing WebSocket payload sizes. Instead of sending entire prop objects, only the specific fields that changed are transmitted using RFC 6902 JSON Patch format. This optimization works seamlessly with complex nested structures, lists, and custom structs through the `LiveVue.Encoder` protocol. It's possible to skip diffs by setting `v-diff` to `false` on the component or by setting `config :live_vue, enable_props_diff: false` in your config. ([#60](https://github.com/Valian/live_vue/pull/60))\n*   **New `useLiveNavigation` Composable**: A new `useLiveNavigation` composable has been added for programmatic navigation, mirroring the functionality of `live_patch` and `live_redirect`. ([#59](https://github.com/Valian/live_vue/pull/59)).\n*   **New `useLiveEvent` Composable**: A new `useLiveEvent` composable has been added to simplify listening to server-pushed events. It automatically manages event listener lifecycle, reducing boilerplate and preventing memory leaks ([#58](https://github.com/Valian/live_vue/pull/58)).\n*   **New `Link` Component**: A new ` ` Vue component has been added to simplify `live_patch` and `live_redirect` navigation within your Vue components. ([#47](https://github.com/Valian/live_vue/pull/47)).\n*   **TypeScript by Default**: The client-side entrypoint at `assets/vue/index.ts` is now a TypeScript file by default, improving type safety and the development experience out of the box.\n*   **`$live` Template Property**: The LiveView hook instance is now automatically available in all Vue templates as the `$live` property, providing a convenient alternative to `useLiveVue()`. The property is now also fully typed, providing autocompletion and type checking in your editor. ([#56](https://github.com/Valian/live_vue/pull/56)).\n*   **Documentation Overhaul**: The documentation has been completely rewritten and expanded. It now includes comprehensive guides on architecture, basic and advanced usage, a full client-side API reference, a component reference, and much more. ([#49](https://github.com/Valian/live_vue/pull/49))\n*   **Testing Utilities**: The new `LiveVue.Test` module provides helpers to inspect Vue component configuration (props, slots, event handlers) within your LiveView tests, making it easier to write assertions. ([#46](https://github.com/Valian/live_vue/pull/46))\n*   **GitHub CI**: A new GitHub Actions workflow has been added to run tests automatically.","ref":"changelog.html#features-and-improvements"},{"type":"extras","title":"⬆️ Migration Guide - Changelog","doc":"This version transitions the default client-side setup to TypeScript and renames ~V sigil to ~VUE. If you have an existing `assets/vue/index.js`, follow these steps to upgrade:\n\n1.  **Rename and replace `index.js`**:\n    *   Delete your existing `assets/vue/index.js`.\n    *   Create a new file at `assets/vue/index.ts` with the following content:\n\n    ```typescript\n    // polyfill recommended by Vite https://vitejs.dev/config/build-options#build-modulepreload\n    import \"vite/modulepreload-polyfill\"\n    import { Component, h } from \"vue\"\n    import { createLiveVue, findComponent, type LiveHook, type ComponentMap } from \"live_vue\"\n\n    // needed to make $live available in the Vue component\n    declare module \"vue\" {\n      interface ComponentCustomProperties {\n        $live: LiveHook\n      }\n    }\n\n    export default createLiveVue({\n      // name will be passed as-is in v-component of the .vue HEEX component\n      resolve: name => {\n        // we're importing from ../../lib to allow collocating Vue files with LiveView files\n        // eager: true disables lazy loading - all these components will be part of the app.js bundle\n        // more: https://vite.dev/guide/features.html#glob-import\n        const components = {\n          ...import.meta.glob(\"./**/*.vue\", { eager: true }),\n          ...import.meta.glob(\"../../lib/**/*.vue\", { eager: true }),\n        } as ComponentMap\n\n        // finds component by name or path suffix and gives a nice error message.\n        // `path/to/component/index.vue` can be found as `path/to/component` or simply `component`\n        // `path/to/Component.vue` can be found as `path/to/Component` or simply `Component`\n        return findComponent(components as ComponentMap, name)\n      },\n      // it's a default implementation of creating and mounting vue app, you can easily extend it to add your own plugins, directives etc.\n      setup: ({ createApp, component, props, slots, plugin, el }) => {\n        const app = createApp({ render: () => h(component as Component, props, slots) })\n        app.use(plugin)\n        // add your own plugins here\n        // app.use(pinia)\n        app.mount(el)\n        return app\n      },\n    })\n    ```\n\n2.  **Update `tsconfig.json`**:\n    Add `vue/index.ts` to the `include` array in your `assets/tsconfig.json` file. The final result should look similar to this:\n\n    ```json\n    {\n      \"compilerOptions\": {\n        // ...\n      },\n      \"include\": [\"js/**/*.ts\", \"js/**/*.js\", \"js/**/*.tsx\", \"vue/**/*.vue\", \"vue/index.ts\"]\n    }\n    ```","ref":"changelog.html#migration-guide"},{"type":"extras","title":"🐛 Bug Fixes - Changelog","doc":"*   **SSR Attribute Rendering**: Fixed a bug where the `data-ssr` attribute was not being rendered correctly as a boolean `true` or `false` in the final HTML.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Housekeeping - Changelog","doc":"*   **Optional Floki Dependency**: `floki` is now an optional dependency, only required if you use the new testing utilities.\n*   **Dependency Updates**: NPM dependencies have been updated to address security vulnerabilities.\n*   **Dropped Elixir 1.12 Support**: Support for Elixir 1.12 has been removed to align with Phoenix LiveView's supported versions.","ref":"changelog.html#housekeeping"},{"type":"extras","title":"0.5.7 - 2024-12-04 - Changelog","doc":"","ref":"changelog.html#0-5-7-2024-12-04"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fix the `useLiveVue` hook typings to show all available functions and properties.","ref":"changelog.html#fixes-1"},{"type":"extras","title":"0.5.6 - 2024-11-27 - Changelog","doc":"","ref":"changelog.html#0-5-6-2024-11-27"},{"type":"extras","title":"Improvements - Changelog","doc":"- Much better typing for the library [#41](https://github.com/Valian/live_vue/pull/41). Big thanks to [@francois-codes](https://github.com/francois-codes) for the contribution!\n- Added `worker` option to `vite.config.js`, and added instruction how to deal with typescript error [#45](https://github.com/Valian/live_vue/pull/45)","ref":"changelog.html#improvements"},{"type":"extras","title":"0.5.5 - 2024-11-14 - Changelog","doc":"","ref":"changelog.html#0-5-5-2024-11-14"},{"type":"extras","title":"Fixes - Changelog","doc":"- Slots are now rendered correctly in SSR [#39](https://github.com/Valian/live_vue/pull/39)","ref":"changelog.html#fixes-2"},{"type":"extras","title":"0.5.4 - 2024-11-13 - Changelog","doc":"","ref":"changelog.html#0-5-4-2024-11-13"},{"type":"extras","title":"Fixed - Changelog","doc":"- added type: module to package.json in live_vue to fix the older nodejs module resolution issue [#36](https://github.com/Valian/live_vue/issues/36)","ref":"changelog.html#fixed"},{"type":"extras","title":"0.5.3 - 2024-11-12 - Changelog","doc":"","ref":"changelog.html#0-5-3-2024-11-12"},{"type":"extras","title":"Fixed - Changelog","doc":"- Added explicit extensions to all JS imports. It should fix some issues with module resulution. [#36](https://github.com/Valian/live_vue/issues/36)","ref":"changelog.html#fixed-1"},{"type":"extras","title":"0.5.2 - 2024-10-08 - Changelog","doc":"","ref":"changelog.html#0-5-2-2024-10-08"},{"type":"extras","title":"Changed - Changelog","doc":"- Added hint to pass `--silent` flag to `npm` watcher in `INSTALLATION.md`. It prevents `npm` from printing executed command which is not useful and makes output messy.\n\n```elixir\nconfig :my_app, MyAppWeb.Endpoint,\n  # ...\n  watchers: [\n    npm: [\"--silent\", \"run\", \"dev\", cd: Path.expand(\"../assets\", __DIR__)]\n  ]\n```","ref":"changelog.html#changed"},{"type":"extras","title":"0.5.1 - 2024-10-08 - Changelog","doc":"","ref":"changelog.html#0-5-1-2024-10-08"},{"type":"extras","title":"Fixed - Changelog","doc":"- Fixed a bug where the server was not preloading the correct assets for the Vue components. It happened because CursorAI \"skipped\" important part of the code when migrating to the TypeScript 😅","ref":"changelog.html#fixed-2"},{"type":"extras","title":"0.5.0 - 2024-10-08 - Changelog","doc":"","ref":"changelog.html#0-5-0-2024-10-08"},{"type":"extras","title":"Changed - Changelog","doc":"- Migrated the project to TypeScript 💜 [#32](https://github.com/Valian/live_vue/pull/32)\n- Added `createLiveVue` entrypoint to make it easier to customize Vue app initialization","ref":"changelog.html#changed-1"},{"type":"extras","title":"Deprecations - Changelog","doc":"- `assets/vue/index.js` should export app created by `createLiveVue()`, not just available components. See migration below.","ref":"changelog.html#deprecations"},{"type":"extras","title":"Migration - Changelog","doc":"In `assets/js/app.js`, instead of:\n\n```js\nexport default {\n  ...import.meta.glob(\"./**/*.vue\", { eager: true }),\n  ...import.meta.glob(\"../../lib/**/*.vue\", { eager: true }),\n}\n```\n\nuse:\n```js\n// polyfill recommended by Vite https://vitejs.dev/config/build-options#build-modulepreload\nimport \"vite/modulepreload-polyfill\"\nimport { h } from \"vue\"\nimport { createLiveVue, findComponent } from \"live_vue\"\n\nexport default createLiveVue({\n  resolve: name => {\n    const components = {\n      ...import.meta.glob(\"./**/*.vue\", { eager: true }),\n      ...import.meta.glob(\"../../lib/**/*.vue\", { eager: true }),\n    }\n\n    // finds component by name or path suffix and gives a nice error message.\n    // `path/to/component/index.vue` can be found as `path/to/component` or simply `component`\n    // `path/to/Component.vue` can be found as `path/to/Component` or simply `Component`\n    return findComponent(components, name)\n  },\n  setup: ({ createApp, component, props, slots, plugin, el }) => {\n    const app = createApp({ render: () => h(component, props, slots) })\n    app.use(plugin)\n    app.mount(el)\n    return app\n  },\n})\n```\n\nthen, in `assets/js/app.js`, instead of:\n\n```js\nimport components from \"./vue\"\n```\n\nsimply do\n\n```js\nimport { getHooks } from \"live_vue\"\nimport liveVueApp from \"./vue\"\n// ...\n\nconst hooks = { ...getHooks(liveVueApp) }\n```\n\nIf you had any custom initialization code, you have to move it to `createLiveVue().setup()` function.","ref":"changelog.html#migration"},{"type":"extras","title":"Fixed - Changelog","doc":"- Nicely formatted JS error stracktraces during SSR [commit](https://github.com/Valian/live_vue/commit/10f672bce4104a38523905c52c4879083e4bc6db)\n- Previously `initializeVueApp` was not working in SSR mode, since it was declared in app.js which couldn't be imported by server bundle. Now it's in a separate file as `createLiveVue().setup()` and can be imported by both client and server bundles.","ref":"changelog.html#fixed-3"},{"type":"extras","title":"0.4.1 - 2024-08-30 - Changelog","doc":"","ref":"changelog.html#0-4-1-2024-08-30"},{"type":"extras","title":"Changed - Changelog","doc":"-   Improved `pathToFullPathAndFilename` to work with `index.vue` files. Now `../ComponentName/index.vue` can be referenced as `ComponentName` [#23](https://github.com/Valian/live_vue/pull/23)","ref":"changelog.html#changed-2"},{"type":"extras","title":"0.4.0 - 2024-06-12 - Changelog","doc":"","ref":"changelog.html#0-4-0-2024-06-12"},{"type":"extras","title":"New feature - Changelog","doc":"-   Support for custom Vue instance initialization [#13](https://github.com/Valian/live_vue/pull/13) by @morfert","ref":"changelog.html#new-feature"},{"type":"extras","title":"0.3.9 - 2024-06-07 - Changelog","doc":"","ref":"changelog.html#0-3-9-2024-06-07"},{"type":"extras","title":"0.3.8 - 2024-06-01 - Changelog","doc":"","ref":"changelog.html#0-3-8-2024-06-01"},{"type":"extras","title":"Fixed - Changelog","doc":"-   Invalid live_vue import in copied package.json (`file:../..` -> `file:../deps/live_vue`)\n-   Changed `useLiveVue` inject key from `Symbol()` to `_live_vue` string, so it's working if Vite does a reload and Symbol is re-evaluated.","ref":"changelog.html#fixed-4"},{"type":"extras","title":"Added - Changelog","doc":"-   Added live_vue, phoenix, phoenix_html and phonenix_live_vue to vite `optimizeDeps.include` config options. It should pre-bundle these packages in development, making it consistent with packages imported from node_modules and improve DX.\n-   Added initial typescript definitions. Apparently it's enough to name them ` .d.mts`, so I've created them both for `index.mjs` and `server.mjs`","ref":"changelog.html#added"},{"type":"extras","title":"0.3.7 - 2024-05-26 - Changelog","doc":"","ref":"changelog.html#0-3-7-2024-05-26"},{"type":"extras","title":"Changed - Changelog","doc":"-   Added a Mix.Task to make JS file setup more straightforward and cross-platform [#11](https://github.com/Valian/live_vue/pull/11). Contribution by @morfert 🔥\n-   Installation instruction was moved to the separate file\n-   Package.json was added to files automatically copied from live_vue when using `mix live_vue.setup`","ref":"changelog.html#changed-3"},{"type":"extras","title":"Fixed - Changelog","doc":"-   Removed `build: {modulePreload: { polyfill: false }}` from vite.config.js as it made it impossible to use `vite/modulepreload-polyfill`. To migrate: please remove that line from yours vite.config.js. Fixed [#12](https://github.com/Valian/live_vue/issues/12)","ref":"changelog.html#fixed-5"},{"type":"extras","title":"0.3.6 - 2024-05-24 - Changelog","doc":"","ref":"changelog.html#0-3-6-2024-05-24"},{"type":"extras","title":"Fixed - Changelog","doc":"-   Fixed missing import in loadManifest\n-   Added `import \"vite/modulepreload-polyfill\";` to `assets/vue/index.js`. To migrate, add that line to the top. It adds polyfill for module preload, required for some browsers. More here: https://vitejs.dev/config/build-options#build-modulepreload","ref":"changelog.html#fixed-6"},{"type":"extras","title":"0.3.5 - 2024-05-24 - Changelog","doc":"","ref":"changelog.html#0-3-5-2024-05-24"},{"type":"extras","title":"Changed - Changelog","doc":"-   Removed `body-parser` dependency from `live_vue`. Should fix [#9](https://github.com/Valian/live_vue/issues/9)","ref":"changelog.html#changed-4"},{"type":"extras","title":"0.3.4 - 2024-05-22 - Changelog","doc":"","ref":"changelog.html#0-3-4-2024-05-22"},{"type":"extras","title":"Fixed - Changelog","doc":"-   Props are correctly updated when being arrays of structs","ref":"changelog.html#fixed-7"},{"type":"extras","title":"0.3.3 - 2024-05-22 - Changelog","doc":"","ref":"changelog.html#0-3-3-2024-05-22"},{"type":"extras","title":"Fixed - Changelog","doc":"-   Javascript imports were mixed - vitePlugin.js was using CJS, rest was using ESM. Now it's explicit by adding \".mjs\" extension.\n-   Removed `:attr` declarations for `<.vue>` component to avoid warnings related to unexpected props being passed to `:rest` attribute [#8](https://github.com/Valian/live_vue/pull/8)","ref":"changelog.html#fixed-8"},{"type":"extras","title":"0.3.2 - 2024-05-19 - Changelog","doc":"","ref":"changelog.html#0-3-2-2024-05-19"},{"type":"extras","title":"Fixed - Changelog","doc":"-   Hot reload of CSS when updating Elixir files","ref":"changelog.html#fixed-9"},{"type":"extras","title":"0.3.1 - 2024-05-17 - Changelog","doc":"","ref":"changelog.html#0-3-1-2024-05-17"},{"type":"extras","title":"Changed - Changelog","doc":"-   Simplified `assets/vue/index.js` file - mapping filenames to keys is done by the library. Previous version should still work.","ref":"changelog.html#changed-5"},{"type":"extras","title":"0.3.0 - 2024-05-17 - Changelog","doc":"","ref":"changelog.html#0-3-0-2024-05-17"},{"type":"extras","title":"CHANGED - Changelog","doc":"-   removed esbuild from live_vue, `package.json` points directly to `assets/js/live_vue`\n-   added support to lazy loading components. See more in README. To migrate, ensure all steps from installation are up-to-date.","ref":"changelog.html#changed-6"},{"type":"extras","title":"0.2.0 - 2024-05-17 - Changelog","doc":"QoL release","ref":"changelog.html#0-2-0-2024-05-17"},{"type":"extras","title":"Added - Changelog","doc":"-   `@` added to Vite & typescript paths. To migrate, see `assets/copy/tsconfig.json` and `assets/copy/vite.config.js`\n-   Added Vite types to tsconfig.json to support special imports, eg. svg. To migrate, add `\"types\": [\"vite/client\"]`.\n-   Added possibility to colocate Vue files in `lib` directory. To migrate, copy `assets/copy/vue/index.js` to your project.","ref":"changelog.html#added-1"},{"type":"extras","title":"Changed - Changelog","doc":"-   Adjusted files hierarchy to match module names\n-   Publishing with expublish\n\n## [0.1.0] - 2024-05-15","ref":"changelog.html#changed-7"},{"type":"extras","title":"Initial release - Changelog","doc":"-   Start of the project\n-   End-To-End Reactivity with LiveView\n-   Server-Side Rendered (SSR) Vue\n-   Tailwind Support\n-   Dead View Support\n-   Vite support","ref":"changelog.html#initial-release"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}